@@ -195,6 +195,9 @@ gst_cea_cc_overlay_get_type (void)
   return type;
 }
 
+GST_ELEMENT_REGISTER_DEFINE (cc708overlay, "cc708overlay",
+    GST_RANK_PRIMARY, GST_TYPE_CEA_CC_OVERLAY);
+
 static void
 gst_base_cea_cc_overlay_base_init (gpointer g_class)
 {
@@ -204,15 +207,9 @@ gst_base_cea_cc_overlay_base_init (gpointer g_class)
   /* Only lock for the subclasses here, the base class
    * doesn't have this mutex yet and it's not necessary
    * here */
-  /* FIXME : Not needed anymore since pango 1.32.6 ! */
-  if (klass->pango_lock)
-    g_mutex_lock (klass->pango_lock);
   fontmap = pango_cairo_font_map_get_default ();
   klass->pango_context =
       pango_font_map_create_context (PANGO_FONT_MAP (fontmap));
-  if (klass->pango_lock)
-    g_mutex_unlock (klass->pango_lock);
-
 }
 
 static void
@@ -243,9 +240,6 @@ gst_base_cea_cc_overlay_class_init (GstCeaCcOverlayClass * klass)
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_cea_cc_overlay_change_state);
 
-  klass->pango_lock = g_slice_new (GMutex);
-  g_mutex_init (klass->pango_lock);
-
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SERVICE_NUMBER,
       g_param_spec_int ("service-number", "service-number",
           "Service number. Service 1 is designated as the Primary Caption Service,"
@@ -285,6 +279,8 @@ gst_base_cea_cc_overlay_class_init (GstCeaCcOverlayClass * klass)
       "Chengjun Wang <cjun.wang@samsung.com>");
   gst_cea708_decoder_init_debug ();
 
+  gst_type_mark_as_plugin_api (GST_TYPE_CC_OVERLAY_WIN_H_POS, 0);
+
 }
 
 static void
@@ -519,14 +515,11 @@ gst_cea_cc_overlay_setcaps (GstCeaCcOverlay * overlay, GstCaps * caps)
   ret = gst_cea_cc_overlay_negotiate (overlay, caps);
 
   GST_CEA_CC_OVERLAY_LOCK (overlay);
-  g_mutex_lock (GST_CEA_CC_OVERLAY_GET_CLASS (overlay)->pango_lock);
   if (!overlay->attach_compo_to_buffer &&
       !gst_cea_cc_overlay_can_handle_caps (caps)) {
     GST_DEBUG_OBJECT (overlay, "unsupported caps %" GST_PTR_FORMAT, caps);
     ret = FALSE;
   }
-
-  g_mutex_unlock (GST_CEA_CC_OVERLAY_GET_CLASS (overlay)->pango_lock);
   GST_CEA_CC_OVERLAY_UNLOCK (overlay);
 
   return ret;
@@ -995,7 +988,6 @@ gst_cea_cc_overlay_cc_event (GstPad * pad, GstObject * parent, GstEvent * event)
             ("received non-TIME newsegment event on text input"));
       }
 
-      gst_event_unref (event);
       ret = TRUE;
 
       /* wake up the video chain, it might be waiting for a text buffer or
@@ -1022,7 +1014,6 @@ gst_cea_cc_overlay_cc_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GST_CEA_CC_OVERLAY_BROADCAST (overlay);
       GST_CEA_CC_OVERLAY_UNLOCK (overlay);
 
-      gst_event_unref (event);
       ret = TRUE;
       break;
     }
@@ -1034,7 +1025,6 @@ gst_cea_cc_overlay_cc_event (GstPad * pad, GstObject * parent, GstEvent * event)
       gst_cea_cc_overlay_pop_text (overlay);
       gst_segment_init (&overlay->cc_segment, GST_FORMAT_TIME);
       GST_CEA_CC_OVERLAY_UNLOCK (overlay);
-      gst_event_unref (event);
       ret = TRUE;
       break;
     case GST_EVENT_FLUSH_START:
@@ -1043,7 +1033,6 @@ gst_cea_cc_overlay_cc_event (GstPad * pad, GstObject * parent, GstEvent * event)
       overlay->cc_flushing = TRUE;
       GST_CEA_CC_OVERLAY_BROADCAST (overlay);
       GST_CEA_CC_OVERLAY_UNLOCK (overlay);
-      gst_event_unref (event);
       ret = TRUE;
       break;
     case GST_EVENT_EOS:
@@ -1054,14 +1043,18 @@ gst_cea_cc_overlay_cc_event (GstPad * pad, GstObject * parent, GstEvent * event)
        * a text segment update */
       GST_CEA_CC_OVERLAY_BROADCAST (overlay);
       GST_CEA_CC_OVERLAY_UNLOCK (overlay);
-      gst_event_unref (event);
       ret = TRUE;
       break;
     default:
-      ret = gst_pad_event_default (pad, parent, event);
       break;
   }
 
+  if (ret) {
+    gst_event_unref (event);
+  } else {
+    ret = gst_pad_event_default (pad, parent, event);
+  }
+
   return ret;
 }
 
@@ -1468,7 +1461,7 @@ gst_cea_cc_overlay_process_packet (GstCeaCcOverlay * overlay, guint8 cc_type)
  * @overlay: The #GstCeaCcOverlay 
  * @user_data: The #GstMpegVideoCCData to decode
  *
- * decode closed caption data and render when neccesary
+ * decode closed caption data and render when necessary
  * in struct GstMpegVideoCCData type's user_data's data field, 3 byte's data construct 1 cc_data_pkt
  *
  * A cc_data_pkt is 3 bytes as follows:
