@@ -21,7 +21,6 @@
  * SECTION:element-curlhttpsink
  * @title: curlhttpsink
  * @short_description: sink that uploads data to a server using libcurl
- * @see_also:
  *
  * This is a network sink that uses libcurl as a client to upload data to
  * an HTTP server.
@@ -65,6 +64,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include "gstcurlelements.h"
 #include "gstcurltlssink.h"
 #include "gstcurlhttpsink.h"
 
@@ -98,6 +98,7 @@ static void gst_curl_http_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_curl_http_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static gboolean gst_curl_http_sink_stop (GstBaseSink * bsink);
 static void gst_curl_http_sink_finalize (GObject * gobject);
 static gboolean gst_curl_http_sink_set_header_unlocked
     (GstCurlBaseSink * bcsink);
@@ -112,7 +113,8 @@ static void gst_curl_http_sink_transfer_prepare_poll_wait
 
 #define gst_curl_http_sink_parent_class parent_class
 G_DEFINE_TYPE (GstCurlHttpSink, gst_curl_http_sink, GST_TYPE_CURL_TLS_SINK);
-
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (curlhttpsink, "curlhttpsink",
+    GST_RANK_NONE, GST_TYPE_CURL_HTTP_SINK, curl_element_init (plugin));
 /* private functions */
 
 static gboolean proxy_setup (GstCurlBaseSink * bcsink);
@@ -122,11 +124,11 @@ gst_curl_http_sink_class_init (GstCurlHttpSinkClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstCurlBaseSinkClass *gstcurlbasesink_class = (GstCurlBaseSinkClass *) klass;
+  GstBaseSinkClass *gstbasesink_class = (GstBaseSinkClass *) klass;
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   GST_DEBUG_CATEGORY_INIT (gst_curl_http_sink_debug, "curlhttpsink", 0,
       "curl http sink element");
-  GST_DEBUG_OBJECT (klass, "class_init");
 
   gst_element_class_set_static_metadata (element_class,
       "Curl http sink",
@@ -144,6 +146,7 @@ gst_curl_http_sink_class_init (GstCurlHttpSinkClass * klass)
   gstcurlbasesink_class->transfer_prepare_poll_wait =
       gst_curl_http_sink_transfer_prepare_poll_wait;
 
+  gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_curl_http_sink_stop);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_curl_http_sink_finalize);
 
   gobject_class->set_property = gst_curl_http_sink_set_property;
@@ -171,7 +174,8 @@ gst_curl_http_sink_class_init (GstCurlHttpSinkClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
       g_param_spec_string ("content-type", "Content type",
-          "The mime type of the body of the request", NULL,
+          "Content Type to use for the Content-Type header. If not set, "
+          "detected mime type will be used", NULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
@@ -181,6 +185,7 @@ gst_curl_http_sink_init (GstCurlHttpSink * sink)
   sink->header_list = NULL;
   sink->use_content_length = DEFAULT_USE_CONTENT_LENGTH;
   sink->content_type = NULL;
+  sink->discovered_content_type = NULL;
 
   sink->proxy_port = DEFAULT_PROXY_PORT;
   sink->proxy_headers_set = FALSE;
@@ -200,6 +205,7 @@ gst_curl_http_sink_finalize (GObject * gobject)
   g_free (this->proxy_user);
   g_free (this->proxy_passwd);
   g_free (this->content_type);
+  g_free (this->discovered_content_type);
 
   if (this->header_list) {
     curl_slist_free_all (this->header_list);
@@ -319,6 +325,8 @@ gst_curl_http_sink_set_header_unlocked (GstCurlBaseSink * bcsink)
 {
   GstCurlHttpSink *sink = GST_CURL_HTTP_SINK (bcsink);
   gchar *tmp;
+  const gchar *used_content_type;
+
   CURLcode res;
 
   if (sink->header_list) {
@@ -336,10 +344,14 @@ gst_curl_http_sink_set_header_unlocked (GstCurlBaseSink * bcsink)
   if (sink->use_content_length) {
     /* if content length is used we assume that every buffer is one
      * entire file, which is the case when uploading several jpegs */
-    tmp =
-        g_strdup_printf ("Content-Length: %d", (int) bcsink->transfer_buf->len);
-    sink->header_list = curl_slist_append (sink->header_list, tmp);
-    g_free (tmp);
+    res =
+        curl_easy_setopt (bcsink->curl, CURLOPT_POSTFIELDSIZE,
+        (long) bcsink->transfer_buf->len);
+    if (res != CURLE_OK) {
+      bcsink->error = g_strdup_printf ("failed to set HTTP content-length: %s",
+          curl_easy_strerror (res));
+      return FALSE;
+    }
   } else {
     /* when sending a POST request to a HTTP 1.1 server, you can send data
      * without knowing the size before starting the POST if you use chunked
@@ -348,9 +360,20 @@ gst_curl_http_sink_set_header_unlocked (GstCurlBaseSink * bcsink)
         "Transfer-Encoding: chunked");
   }
 
-  tmp = g_strdup_printf ("Content-Type: %s", sink->content_type);
-  sink->header_list = curl_slist_append (sink->header_list, tmp);
-  g_free (tmp);
+  if (sink->content_type)
+    used_content_type = sink->content_type;
+  else
+    used_content_type = sink->discovered_content_type;
+
+  if (used_content_type) {
+    tmp = g_strdup_printf ("Content-Type: %s", used_content_type);
+    sink->header_list = curl_slist_append (sink->header_list, tmp);
+    g_free (tmp);
+  } else {
+    GST_WARNING_OBJECT (sink,
+        "No content-type available to set in header, continue without it");
+  }
+
 
 set_headers:
 
@@ -360,6 +383,13 @@ set_headers:
     sink->header_list = curl_slist_append (sink->header_list, tmp);
     g_free (tmp);
   }
+
+  /* set 'Expect: 100-continue'-header explicitly */
+  if (sink->use_content_length) {
+    sink->header_list =
+        curl_slist_append (sink->header_list, "Expect: 100-continue");
+  }
+
   res = curl_easy_setopt (bcsink->curl, CURLOPT_HTTPHEADER, sink->header_list);
   if (res != CURLE_OK) {
     bcsink->error = g_strdup_printf ("failed to set HTTP headers: %s",
@@ -420,8 +450,9 @@ gst_curl_http_sink_transfer_verify_response_code (GstCurlBaseSink * bcsink)
   GST_DEBUG_OBJECT (sink, "response code: %ld", resp);
 
   if (resp < 100 || resp >= 300) {
-    bcsink->error = g_strdup_printf ("HTTP response error: (received: %ld)",
-        resp);
+    GST_ELEMENT_ERROR_WITH_DETAILS (sink, RESOURCE, WRITE,
+        ("HTTP response error code: %ld", resp), (NULL), ("http-status-code",
+            G_TYPE_UINT, (guint) resp, NULL));
     return FALSE;
   }
 
@@ -463,18 +494,31 @@ gst_curl_http_sink_set_mime_type (GstCurlBaseSink * bcsink, GstCaps * caps)
   structure = gst_caps_get_structure (caps, 0);
   mime_type = gst_structure_get_name (structure);
 
-  g_free (sink->content_type);
+  g_free (sink->discovered_content_type);
   if (!g_strcmp0 (mime_type, "multipart/form-data") &&
       gst_structure_has_field_typed (structure, "boundary", G_TYPE_STRING)) {
     const gchar *boundary;
 
     boundary = gst_structure_get_string (structure, "boundary");
-    sink->content_type = g_strconcat (mime_type, "; boundary=", boundary, NULL);
+    sink->discovered_content_type =
+        g_strconcat (mime_type, "; boundary=", boundary, NULL);
   } else {
-    sink->content_type = g_strdup (mime_type);
+    sink->discovered_content_type = g_strdup (mime_type);
   }
 }
 
+static gboolean
+gst_curl_http_sink_stop (GstBaseSink * bsink)
+{
+  gboolean ret;
+  GstCurlHttpSink *sink = GST_CURL_HTTP_SINK (bsink);
+  ret = GST_BASE_SINK_CLASS (parent_class)->stop (bsink);
+
+  g_clear_pointer (&sink->discovered_content_type, g_free);
+
+  return ret;
+}
+
 static gboolean
 proxy_setup (GstCurlBaseSink * bcsink)
 {
