@@ -100,6 +100,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_dash_sink_debug);
  * GstDashSinkMuxerType:
  * @GST_DASH_SINK_MUXER_TS: Use mpegtsmux
  * @GST_DASH_SINK_MUXER_MP4: Use mp4mux
+ * @GST_DASH_SINK_MUXER_DASHMP4: Use dashmp4mux
  *
  * Muxer type
  */
@@ -107,6 +108,7 @@ typedef enum
 {
   GST_DASH_SINK_MUXER_TS = 0,
   GST_DASH_SINK_MUXER_MP4 = 1,
+  GST_DASH_SINK_MUXER_DASHMP4 = 2,
 } GstDashSinkMuxerType;
 
 typedef struct _DashSinkMuxer
@@ -124,7 +126,14 @@ gst_dash_sink_muxer_get_type (void)
   static GType dash_sink_muxer_type = 0;
   static const GEnumValue muxer_type[] = {
     {GST_DASH_SINK_MUXER_TS, "Use mpegtsmux", "ts"},
-    {GST_DASH_SINK_MUXER_MP4, "Use mp4mux", "mp4"},
+    {GST_DASH_SINK_MUXER_MP4, "Use mp4mux (deprecated, non-functional)", "mp4"},
+    /**
+     * GstDashSinkMuxerType::dashmp4
+     *
+     *
+     * Since: 1.24
+     */
+    {GST_DASH_SINK_MUXER_DASHMP4, "Use dashmp4mux", "dashmp4"},
     {0, NULL, NULL},
   };
 
@@ -146,6 +155,11 @@ static const DashSinkMuxer dash_muxer_list[] = {
         "mp4mux",
         "video/mp4",
       "mp4"},
+  {
+        GST_DASH_SINK_MUXER_DASHMP4,
+        "dashmp4mux",
+        "video/mp4",
+      "mp4"},
 };
 
 #define DEFAULT_SEGMENT_LIST_TPL "_%05d"
@@ -160,6 +174,7 @@ static const DashSinkMuxer dash_muxer_list[] = {
 #define DEFAULT_MPD_USE_SEGMENT_LIST FALSE
 #define DEFAULT_MPD_MIN_BUFFER_TIME 2000
 #define DEFAULT_MPD_PERIOD_DURATION GST_CLOCK_TIME_NONE
+#define DEFAULT_MPD_SUGGESTED_PRESENTATION_DELAY 0
 
 #define DEFAULT_DASH_SINK_MUXER GST_DASH_SINK_MUXER_TS
 
@@ -184,6 +199,7 @@ enum
   PROP_MPD_MIN_BUFFER_TIME,
   PROP_MPD_BASEURL,
   PROP_MPD_PERIOD_DURATION,
+  PROP_MPD_SUGGESTED_PRESENTATION_DELAY,
 };
 
 enum
@@ -248,6 +264,7 @@ struct _GstDashSink
   guint index;
   GList *streams;
   guint64 minimum_update_period;
+  guint64 suggested_presentation_delay;
   guint64 min_buffer_time;
   gint64 period_duration;
 };
@@ -539,6 +556,21 @@ gst_dash_sink_class_init (GstDashSinkClass * klass)
           G_MAXUINT64, DEFAULT_MPD_PERIOD_DURATION,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstDashSink:suggested-presentation-delay
+   *
+   * set suggested presentation delay of MPD file in milliseconds
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class,
+      PROP_MPD_SUGGESTED_PRESENTATION_DELAY,
+      g_param_spec_uint64 ("suggested-presentation-delay",
+          "suggested presentation delay",
+          "Provides to the manifest a suggested presentation delay in milliseconds",
+          0, G_MAXUINT64, DEFAULT_MPD_SUGGESTED_PRESENTATION_DELAY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstDashSink::get-playlist-stream:
    * @sink: the #GstDashSink
@@ -628,9 +660,13 @@ gst_dash_sink_add_splitmuxsink (GstDashSink * sink, GstDashSinkStream * stream)
       gst_element_factory_make (dash_muxer_list[sink->muxer].element_name,
       NULL);
 
-  if (sink->muxer == GST_DASH_SINK_MUXER_MP4)
+  if (sink->muxer == GST_DASH_SINK_MUXER_MP4) {
     g_object_set (mux, "fragment-duration", sink->target_duration * GST_MSECOND,
         NULL);
+  } else if (sink->muxer == GST_DASH_SINK_MUXER_DASHMP4) {
+    g_object_set (mux, "fragment-duration", sink->target_duration * GST_SECOND,
+        NULL);
+  }
 
   g_return_val_if_fail (mux != NULL, FALSE);
 
@@ -657,9 +693,12 @@ gst_dash_sink_add_splitmuxsink (GstDashSink * sink, GstDashSinkStream * stream)
   g_object_set (stream->splitmuxsink, "location", NULL,
       "max-size-time", ((GstClockTime) sink->target_duration * GST_SECOND),
       "send-keyframe-requests", TRUE, "muxer", mux, "sink",
-      stream->giostreamsink, "reset-muxer", FALSE, "send-keyframe-requests",
+      stream->giostreamsink, "send-keyframe-requests",
       sink->send_keyframe_requests, NULL);
 
+  if (sink->muxer == GST_DASH_SINK_MUXER_TS)
+    g_object_set (stream->splitmuxsink, "reset-muxer", FALSE, NULL);
+
   g_signal_connect (stream->splitmuxsink, "format-location",
       G_CALLBACK (on_format_location), stream);
 
@@ -680,6 +719,7 @@ gst_dash_sink_init (GstDashSink * sink)
 
   sink->min_buffer_time = DEFAULT_MPD_MIN_BUFFER_TIME;
   sink->period_duration = DEFAULT_MPD_PERIOD_DURATION;
+  sink->suggested_presentation_delay = DEFAULT_MPD_SUGGESTED_PRESENTATION_DELAY;
 
   g_mutex_init (&sink->mpd_lock);
 
@@ -704,23 +744,20 @@ gst_dash_sink_get_stream_metadata (GstDashSink * sink,
   GST_DEBUG_OBJECT (sink, "stream caps %s", gst_caps_to_string (caps));
   s = gst_caps_get_structure (caps, 0);
 
+  g_free (stream->codec);
+  stream->codec = gst_codec_utils_caps_get_mime_codec (caps);
+
   switch (stream->type) {
     case DASH_SINK_STREAM_TYPE_VIDEO:
     {
       gst_structure_get_int (s, "width", &stream->info.video.width);
       gst_structure_get_int (s, "height", &stream->info.video.height);
-      g_free (stream->codec);
-      stream->codec =
-          g_strdup (gst_mpd_helper_get_video_codec_from_mime (caps));
       break;
     }
     case DASH_SINK_STREAM_TYPE_AUDIO:
     {
       gst_structure_get_int (s, "channels", &stream->info.audio.channels);
       gst_structure_get_int (s, "rate", &stream->info.audio.rate);
-      g_free (stream->codec);
-      stream->codec =
-          g_strdup (gst_mpd_helper_get_audio_codec_from_mime (caps));
       break;
     }
     case DASH_SINK_STREAM_TYPE_SUBTITLE:
@@ -756,6 +793,10 @@ gst_dash_sink_generate_mpd_content (GstDashSink * sink,
     if (sink->minimum_update_period)
       gst_mpd_client_set_root_node (sink->mpd_client,
           "minimum-update-period", sink->minimum_update_period, NULL);
+    if (sink->suggested_presentation_delay)
+      gst_mpd_client_set_root_node (sink->mpd_client,
+          "suggested-presentation-delay", sink->suggested_presentation_delay,
+          NULL);
     if (sink->mpd_baseurl)
       gst_mpd_client_add_baseurl_node (sink->mpd_client, "url",
           sink->mpd_baseurl, NULL);
@@ -1111,6 +1152,9 @@ gst_dash_sink_set_property (GObject * object, guint prop_id,
     case PROP_MPD_MINIMUM_UPDATE_PERIOD:
       sink->minimum_update_period = g_value_get_uint64 (value);
       break;
+    case PROP_MPD_SUGGESTED_PRESENTATION_DELAY:
+      sink->suggested_presentation_delay = g_value_get_uint64 (value);
+      break;
     case PROP_MPD_MIN_BUFFER_TIME:
       sink->min_buffer_time = g_value_get_uint64 (value);
       break;
@@ -1157,6 +1201,9 @@ gst_dash_sink_get_property (GObject * object, guint prop_id,
     case PROP_MPD_MINIMUM_UPDATE_PERIOD:
       g_value_set_uint64 (value, sink->minimum_update_period);
       break;
+    case PROP_MPD_SUGGESTED_PRESENTATION_DELAY:
+      g_value_set_uint64 (value, sink->suggested_presentation_delay);
+      break;
     case PROP_MPD_MIN_BUFFER_TIME:
       g_value_set_uint64 (value, sink->min_buffer_time);
       break;
