@@ -101,13 +101,14 @@ struct _GstDtlsConnectionPrivate
   GstDtlsConnectionSendCallback send_callback;
   gpointer send_callback_user_data;
   GDestroyNotify send_callback_destroy_notify;
+  GstFlowReturn syscall_flow_return;
 
   gboolean timeout_pending;
   GThreadPool *thread_pool;
 };
 
-G_DEFINE_TYPE_WITH_CODE (GstDtlsConnection, gst_dtls_connection, G_TYPE_OBJECT,
-    G_ADD_PRIVATE (GstDtlsConnection)
+G_DEFINE_TYPE_WITH_CODE (GstDtlsConnection, gst_dtls_connection,
+    GST_TYPE_OBJECT, G_ADD_PRIVATE (GstDtlsConnection)
     GST_DEBUG_CATEGORY_INIT (gst_dtls_connection_debug, "dtlsconnection", 0,
         "DTLS Connection"));
 
@@ -600,6 +601,14 @@ gst_dtls_connection_set_send_callback (GstDtlsConnection * self,
   g_mutex_unlock (&priv->mutex);
 }
 
+void
+gst_dtls_connection_set_flow_return (GstDtlsConnection * self,
+    GstFlowReturn flow_ret)
+{
+  g_return_if_fail (GST_IS_DTLS_CONNECTION (self));
+  self->priv->syscall_flow_return = flow_ret;
+}
+
 GstFlowReturn
 gst_dtls_connection_process (GstDtlsConnection * self, gpointer data, gsize len,
     gsize * written, GError ** err)
@@ -823,13 +832,13 @@ log_state (GstDtlsConnection * self, const gchar * str)
   GstDtlsConnectionPrivate *priv = self->priv;
   guint states = 0;
 
-  states |= (! !SSL_is_init_finished (priv->ssl) << 0);
-  states |= (! !SSL_in_init (priv->ssl) << 4);
-  states |= (! !SSL_in_before (priv->ssl) << 8);
-  states |= (! !SSL_in_connect_init (priv->ssl) << 12);
-  states |= (! !SSL_in_accept_init (priv->ssl) << 16);
-  states |= (! !SSL_want_write (priv->ssl) << 20);
-  states |= (! !SSL_want_read (priv->ssl) << 24);
+  states |= (!!SSL_is_init_finished (priv->ssl) << 0);
+  states |= (!!SSL_in_init (priv->ssl) << 4);
+  states |= (!!SSL_in_before (priv->ssl) << 8);
+  states |= (!!SSL_in_connect_init (priv->ssl) << 12);
+  states |= (!!SSL_in_accept_init (priv->ssl) << 16);
+  states |= (!!SSL_want_write (priv->ssl) << 20);
+  states |= (!!SSL_want_read (priv->ssl) << 24);
 
 #if OPENSSL_VERSION_NUMBER < 0x10100001L
   GST_LOG_OBJECT (self, "%s: role=%s buf=(%d,%p:%d/%d) %x|%x %s",
@@ -1002,13 +1011,19 @@ handle_error (GstDtlsConnection * self, int ret, GstResourceError error_type,
     case SSL_ERROR_WANT_WRITE:
       GST_LOG_OBJECT (self, "SSL wants write");
       return GST_FLOW_OK;
-    case SSL_ERROR_SYSCALL:
+    case SSL_ERROR_SYSCALL:{
+      GstFlowReturn rc = GST_FLOW_OK;
       /* OpenSSL shouldn't be making real system calls, so we can safely
        * ignore syscall errors. System interactions should happen through
        * our BIO.
        */
-      GST_DEBUG_OBJECT (self, "OpenSSL reported a syscall error, ignoring.");
-      return GST_FLOW_OK;
+      if (error_type == GST_RESOURCE_ERROR_WRITE) {
+        rc = self->priv->syscall_flow_return;
+      }
+      GST_DEBUG_OBJECT (self,
+          "OpenSSL reported a syscall error. flow_return=%i", rc);
+      return rc;
+    }
     default:
       if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED) {
         self->priv->connection_state = GST_DTLS_CONNECTION_STATE_FAILED;
@@ -1182,6 +1197,7 @@ bio_method_write (BIO * bio, const char *data, int size)
   gboolean ret = TRUE;
 
   GST_LOG_OBJECT (self, "BIO: writing %d", size);
+  self->priv->syscall_flow_return = GST_FLOW_OK;
 
   if (self->priv->send_callback)
     ret = self->priv->send_callback (self, data, size,
