@@ -26,13 +26,15 @@
 #include <glib.h>
 #include <string.h>
 
-#include "gsthls.h"
 #include "m3u8.h"
+#include "gsthlselements.h"
 
 #define GST_CAT_DEFAULT hls_debug
 
 static GstM3U8MediaFile *gst_m3u8_media_file_new (gchar * uri,
-    gchar * title, GstClockTime duration, guint sequence);
+    gchar * title, GstClockTime duration, guint sequence,
+    GstDateTime * program_dt);
+static void gst_m3u8_init_file_unref (GstM3U8InitFile * self);
 static gchar *uri_join (const gchar * uri, const gchar * path);
 
 GstM3U8 *
@@ -108,13 +110,14 @@ gst_m3u8_unref (GstM3U8 * self)
     g_list_free (self->files);
 
     g_free (self->last_data);
+    g_mutex_clear (&self->lock);
     g_free (self);
   }
 }
 
 static GstM3U8MediaFile *
 gst_m3u8_media_file_new (gchar * uri, gchar * title, GstClockTime duration,
-    guint sequence)
+    guint sequence, GstDateTime * program_dt)
 {
   GstM3U8MediaFile *file;
 
@@ -124,6 +127,7 @@ gst_m3u8_media_file_new (gchar * uri, gchar * title, GstClockTime duration,
   file->duration = duration;
   file->sequence = sequence;
   file->ref_count = 1;
+  file->program_dt = program_dt;
 
   return file;
 }
@@ -143,9 +147,45 @@ gst_m3u8_media_file_unref (GstM3U8MediaFile * self)
   g_return_if_fail (self != NULL && self->ref_count > 0);
 
   if (g_atomic_int_dec_and_test (&self->ref_count)) {
+    if (self->init_file)
+      gst_m3u8_init_file_unref (self->init_file);
     g_free (self->title);
     g_free (self->uri);
     g_free (self->key);
+    if (self->program_dt)
+      gst_date_time_unref (self->program_dt);
+    g_free (self);
+  }
+}
+
+static GstM3U8InitFile *
+gst_m3u8_init_file_new (gchar * uri)
+{
+  GstM3U8InitFile *file;
+
+  file = g_new0 (GstM3U8InitFile, 1);
+  file->uri = uri;
+  file->ref_count = 1;
+
+  return file;
+}
+
+static GstM3U8InitFile *
+gst_m3u8_init_file_ref (GstM3U8InitFile * ifile)
+{
+  g_assert (ifile != NULL && ifile->ref_count > 0);
+
+  g_atomic_int_add (&ifile->ref_count, 1);
+  return ifile;
+}
+
+static void
+gst_m3u8_init_file_unref (GstM3U8InitFile * self)
+{
+  g_return_if_fail (self != NULL && self->ref_count > 0);
+
+  if (g_atomic_int_dec_and_test (&self->ref_count)) {
+    g_free (self->uri);
     g_free (self);
   }
 }
@@ -306,6 +346,140 @@ gst_hls_variant_stream_compare_by_bitrate (gconstpointer a, gconstpointer b)
   return vs_a->bandwidth - vs_b->bandwidth;
 }
 
+/* If we have MEDIA-SEQUENCE, ensure that it's consistent. If it is not,
+ * the client SHOULD halt playback (6.3.4), which is what we do then. */
+static gboolean
+check_media_seqnums (GstM3U8 * self, GList * previous_files)
+{
+  GList *l, *m;
+  GstM3U8MediaFile *f1 = NULL, *f2 = NULL;
+
+  g_return_val_if_fail (previous_files, FALSE);
+
+  if (!self->files) {
+    /* Empty playlists are trivially consistent */
+    return TRUE;
+  }
+
+  /* Find first case of higher/equal sequence number in new playlist.
+   * From there on we can linearly step ahead */
+  for (l = self->files; l; l = l->next) {
+    gboolean match = FALSE;
+
+    f1 = l->data;
+    for (m = previous_files; m; m = m->next) {
+      f2 = m->data;
+
+      if (f1->sequence >= f2->sequence) {
+        match = TRUE;
+        break;
+      }
+    }
+    if (match)
+      break;
+  }
+
+  /* We must have seen at least one entry on each list */
+  g_assert (f1 != NULL);
+  g_assert (f2 != NULL);
+
+  if (!l) {
+    /* No match, no sequence in the new playlist was higher than
+     * any in the old. This is bad! */
+    GST_ERROR ("Media sequence doesn't continue: last new %" G_GINT64_FORMAT
+        " < last old %" G_GINT64_FORMAT, f1->sequence, f2->sequence);
+    return FALSE;
+  }
+
+  for (; l && m; l = l->next, m = m->next) {
+    f1 = l->data;
+    f2 = m->data;
+
+    if (f1->sequence == f2->sequence && !g_str_equal (f1->uri, f2->uri)) {
+      /* Same sequence, different URI. This is bad! */
+      GST_ERROR ("Media URIs inconsistent (sequence %" G_GINT64_FORMAT
+          "): had '%s', got '%s'", f1->sequence, f2->uri, f1->uri);
+      return FALSE;
+    } else if (f1->sequence < f2->sequence) {
+      /* Not same sequence but by construction sequence must be higher in the
+       * new one. All good in that case, if it isn't then this means that
+       * sequence numbers are decreasing or files were inserted */
+      GST_ERROR ("Media sequences inconsistent: %" G_GINT64_FORMAT " < %"
+          G_GINT64_FORMAT ": URIs new '%s' old '%s'", f1->sequence,
+          f2->sequence, f1->uri, f2->uri);
+      return FALSE;
+    }
+  }
+
+  /* All good if we're getting here */
+  return TRUE;
+}
+
+/* If we don't have MEDIA-SEQUENCE, we check URIs in the previous and
+ * current playlist to calculate the/a correct MEDIA-SEQUENCE for the new
+ * playlist in relation to the old. That is, same URIs get the same number
+ * and later URIs get higher numbers */
+static void
+generate_media_seqnums (GstM3U8 * self, GList * previous_files)
+{
+  GList *l, *m;
+  GstM3U8MediaFile *f1 = NULL, *f2 = NULL;
+  gint64 mediasequence;
+
+  g_return_if_fail (previous_files);
+
+  /* Find first case of same URI in new playlist.
+   * From there on we can linearly step ahead */
+  for (l = self->files; l; l = l->next) {
+    gboolean match = FALSE;
+
+    f1 = l->data;
+    for (m = previous_files; m; m = m->next) {
+      f2 = m->data;
+
+      if (g_str_equal (f1->uri, f2->uri)) {
+        match = TRUE;
+        break;
+      }
+    }
+
+    if (match)
+      break;
+  }
+
+  if (l) {
+    /* Match, check that all following ones are matching too and continue
+     * sequence numbers from there on */
+
+    mediasequence = f2->sequence;
+
+    for (; l && m; l = l->next, m = m->next) {
+      f1 = l->data;
+      f2 = m->data;
+
+      f1->sequence = mediasequence;
+      mediasequence++;
+
+      if (!g_str_equal (f1->uri, f2->uri)) {
+        GST_WARNING ("Inconsistent URIs after playlist update: '%s' != '%s'",
+            f1->uri, f2->uri);
+      }
+    }
+  } else {
+    /* No match, this means f2 is the last item in the previous playlist
+     * and we have to start our new playlist at that sequence */
+    mediasequence = f2->sequence + 1;
+    l = self->files;
+  }
+
+  for (; l; l = l->next) {
+    f1 = l->data;
+
+    f1->sequence = mediasequence;
+    mediasequence++;
+  }
+}
+
 /*
  * @data: a m3u8 playlist text data, taking ownership
  */
@@ -313,6 +487,7 @@ gboolean
 gst_m3u8_update (GstM3U8 * self, gchar * data)
 {
   gint val;
+  GstDateTime *program_dt = NULL;
   GstClockTime duration;
   gchar *title, *end;
   gboolean discontinuity = FALSE;
@@ -321,6 +496,9 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
   guint8 iv[16] = { 0, };
   gint64 size = -1, offset = -1;
   gint64 mediasequence;
+  GList *previous_files = NULL;
+  gboolean have_mediasequence = FALSE;
+  GstM3U8InitFile *last_init_file = NULL;
 
   g_return_val_if_fail (self != NULL, FALSE);
   g_return_val_if_fail (data != NULL, FALSE);
@@ -354,11 +532,8 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
   self->last_data = data;
 
   self->current_file = NULL;
-  if (self->files) {
-    g_list_foreach (self->files, (GFunc) gst_m3u8_media_file_unref, NULL);
-    g_list_free (self->files);
-    self->files = NULL;
-  }
+  previous_files = self->files;
+  self->files = NULL;
   self->duration = GST_CLOCK_TIME_NONE;
   mediasequence = 0;
 
@@ -388,7 +563,10 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
       data = uri_join (self->base_uri ? self->base_uri : self->uri, data);
       if (data != NULL) {
         GstM3U8MediaFile *file;
-        file = gst_m3u8_media_file_new (data, title, duration, mediasequence++);
+        file =
+            gst_m3u8_media_file_new (data, title, duration,
+            mediasequence++, program_dt);
+        program_dt = NULL;
 
         /* set encryption params */
         file->key = current_key ? g_strdup (current_key) : NULL;
@@ -421,6 +599,8 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
         }
 
         file->discont = discontinuity;
+        if (last_init_file)
+          file->init_file = gst_m3u8_init_file_ref (last_init_file);
 
         duration = 0;
         title = NULL;
@@ -461,13 +641,26 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
         if (int_from_string (data + 22, &data, &val))
           self->targetduration = val * GST_SECOND;
       } else if (g_str_has_prefix (data_ext_x, "MEDIA-SEQUENCE:")) {
-        if (int_from_string (data + 22, &data, &val))
+        if (int_from_string (data + 22, &data, &val)) {
           mediasequence = val;
+          have_mediasequence = TRUE;
+        }
+      } else if (g_str_has_prefix (data_ext_x, "DISCONTINUITY-SEQUENCE:")) {
+        if (int_from_string (data + 30, &data, &val)
+            && val != self->discont_sequence) {
+          self->discont_sequence = val;
+          discontinuity = TRUE;
+        }
       } else if (g_str_has_prefix (data_ext_x, "DISCONTINUITY")) {
+        self->discont_sequence++;
         discontinuity = TRUE;
       } else if (g_str_has_prefix (data_ext_x, "PROGRAM-DATE-TIME:")) {
-        /* <YYYY-MM-DDThh:mm:ssZ> */
-        GST_DEBUG ("FIXME parse date");
+        if (program_dt)
+          gst_date_time_unref (program_dt);
+        program_dt = gst_date_time_new_from_iso8601_string (data + 25);
+        if (!program_dt) {
+          GST_WARNING ("Could not parse program date/time");
+        }
       } else if (g_str_has_prefix (data_ext_x, "ALLOW-CACHE:")) {
         self->allowcache = g_ascii_strcasecmp (data + 19, "YES") == 0;
       } else if (g_str_has_prefix (data_ext_x, "KEY:")) {
@@ -529,6 +722,47 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
         } else {
           goto next_line;
         }
+      } else if (g_str_has_prefix (data_ext_x, "MAP:")) {
+        gchar *v, *a, *header_uri = NULL;
+
+        data = data + 11;
+
+        while (data != NULL && parse_attributes (&data, &a, &v)) {
+          if (strcmp (a, "URI") == 0) {
+            header_uri =
+                uri_join (self->base_uri ? self->base_uri : self->uri, v);
+          } else if (strcmp (a, "BYTERANGE") == 0) {
+            if (int64_from_string (v, &v, &size)) {
+              if (*v == '@' && !int64_from_string (v + 1, &v, &offset)) {
+                g_free (header_uri);
+                goto next_line;
+              }
+            } else {
+              g_free (header_uri);
+              goto next_line;
+            }
+          }
+        }
+
+        if (header_uri) {
+          GstM3U8InitFile *init_file;
+          init_file = gst_m3u8_init_file_new (header_uri);
+
+          if (size != -1) {
+            init_file->size = size;
+            if (offset != -1)
+              init_file->offset = offset;
+            else
+              init_file->offset = 0;
+          } else {
+            init_file->size = -1;
+            init_file->offset = 0;
+          }
+          if (last_init_file)
+            gst_m3u8_init_file_unref (last_init_file);
+
+          last_init_file = init_file;
+        }
       } else {
         GST_LOG ("Ignored line: %s", data);
       }
@@ -545,22 +779,63 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
   g_free (current_key);
   current_key = NULL;
 
+  if (program_dt) {
+    gst_date_time_unref (program_dt);
+    program_dt = NULL;
+  }
+
+  self->files = g_list_reverse (self->files);
+
+  if (last_init_file)
+    gst_m3u8_init_file_unref (last_init_file);
+
+  if (previous_files) {
+    gboolean consistent = TRUE;
+
+    if (have_mediasequence) {
+      consistent = check_media_seqnums (self, previous_files);
+    } else {
+      generate_media_seqnums (self, previous_files);
+    }
+
+    g_list_foreach (previous_files, (GFunc) gst_m3u8_media_file_unref, NULL);
+    g_list_free (previous_files);
+    previous_files = NULL;
+
+    /* error was reported above already */
+    if (!consistent) {
+      GST_M3U8_UNLOCK (self);
+      return FALSE;
+    }
+  }
+
   if (self->files == NULL) {
     GST_ERROR ("Invalid media playlist, it does not contain any media files");
     GST_M3U8_UNLOCK (self);
     return FALSE;
   }
 
-  self->files = g_list_reverse (self->files);
-
   /* calculate the start and end times of this media playlist. */
   {
     GList *walk;
     GstM3U8MediaFile *file;
     GstClockTime duration = 0;
 
+    mediasequence = -1;
+
     for (walk = self->files; walk; walk = walk->next) {
       file = walk->data;
+
+      if (mediasequence == -1) {
+        mediasequence = file->sequence;
+      } else if (mediasequence >= file->sequence) {
+        GST_ERROR ("Non-increasing media sequence");
+        GST_M3U8_UNLOCK (self);
+        return FALSE;
+      } else {
+        mediasequence = file->sequence;
+      }
+
       duration += file->duration;
       if (file->sequence > self->highest_sequence_number) {
         if (self->highest_sequence_number >= 0) {
@@ -592,22 +867,30 @@ gst_m3u8_update (GstM3U8 * self, gchar * data)
 
     if (GST_M3U8_IS_LIVE (self)) {
       gint i;
+      GstClockTime sequence_pos = 0;
 
       file = g_list_last (self->files);
 
+      if (self->last_file_end >= GST_M3U8_MEDIA_FILE (file->data)->duration) {
+        sequence_pos =
+            self->last_file_end - GST_M3U8_MEDIA_FILE (file->data)->duration;
+      }
+
       /* for live streams, start GST_M3U8_LIVE_MIN_FRAGMENT_DISTANCE from
-       * the end of the playlist. See section 6.3.3 of HLS draft. Note
-       * the -1, because GST_M3U8_LIVE_MIN_FRAGMENT_DISTANCE = 1 means
-       * start 1 target-duration from the end */
-      for (i = 0; i < GST_M3U8_LIVE_MIN_FRAGMENT_DISTANCE - 1 && file->prev;
-          ++i)
+       * the end of the playlist. See section 6.3.3 of HLS draft */
+      for (i = 0; i < GST_M3U8_LIVE_MIN_FRAGMENT_DISTANCE && file->prev &&
+          GST_M3U8_MEDIA_FILE (file->prev->data)->duration <= sequence_pos;
+          ++i) {
         file = file->prev;
+        sequence_pos -= GST_M3U8_MEDIA_FILE (file->data)->duration;
+      }
+      self->sequence_position = sequence_pos;
     } else {
       file = g_list_first (self->files);
+      self->sequence_position = 0;
     }
     self->current_file = file;
     self->sequence = GST_M3U8_MEDIA_FILE (file->data)->sequence;
-    self->sequence_position = 0;
     GST_DEBUG ("first sequence: %u", (guint) self->sequence);
   }
 
@@ -653,7 +936,8 @@ m3u8_find_next_fragment (GstM3U8 * m3u8, gboolean forward)
 
 GstM3U8MediaFile *
 gst_m3u8_get_next_fragment (GstM3U8 * m3u8, gboolean forward,
-    GstClockTime * sequence_position, gboolean * discont)
+    GstClockTime * sequence_position, GstDateTime ** program_dt,
+    gboolean * discont)
 {
   GstM3U8MediaFile *file = NULL;
 
@@ -679,6 +963,11 @@ gst_m3u8_get_next_fragment (GstM3U8 * m3u8, gboolean forward,
 
   if (sequence_position)
     *sequence_position = m3u8->sequence_position;
+
+  if (program_dt)
+    *program_dt =
+        file->program_dt ? gst_date_time_ref (file->program_dt) : NULL;
+
   if (discont)
     *discont = file->discont || (m3u8->sequence != file->sequence);
 
@@ -978,7 +1267,7 @@ gst_m3u8_get_seek_range (GstM3U8 * m3u8, gint64 * start, gint64 * stop)
   }
   count = g_list_length (m3u8->files);
 
-  for (walk = m3u8->files; walk && count >= min_distance; walk = walk->next) {
+  for (walk = m3u8->files; walk && count > min_distance; walk = walk->next) {
     file = walk->data;
     --count;
     duration += file->duration;
@@ -1009,9 +1298,12 @@ gst_hls_media_unref (GstHLSMedia * media)
 {
   g_assert (media != NULL && media->ref_count > 0);
   if (g_atomic_int_dec_and_test (&media->ref_count)) {
+    if (media->playlist)
+      gst_m3u8_unref (media->playlist);
     g_free (media->group_id);
     g_free (media->name);
     g_free (media->uri);
+    g_free (media->lang);
     g_free (media);
   }
 }
@@ -1031,9 +1323,9 @@ gst_m3u8_get_hls_media_type_from_string (const gchar * type_name)
   return GST_HLS_MEDIA_TYPE_INVALID;
 }
 
-#define GST_HLS_MEDIA_TYPE_NAME(mtype) gst_m3u8_hls_media_type_get_nick(mtype)
-static inline const gchar *
-gst_m3u8_hls_media_type_get_nick (GstHLSMediaType mtype)
+#define GST_HLS_MEDIA_TYPE_NAME(mtype) gst_hls_media_type_get_name(mtype)
+const gchar *
+gst_hls_media_type_get_name (GstHLSMediaType mtype)
 {
   static const gchar *nicks[GST_HLS_N_MEDIA_TYPES] = { "audio", "video",
     "subtitle", "closed-captions"
@@ -1065,13 +1357,13 @@ gst_m3u8_unquote (const gchar * str)
 static GstHLSMedia *
 gst_m3u8_parse_media (gchar * desc, const gchar * base_uri)
 {
-  GstHLSMediaType mtype = GST_HLS_MEDIA_TYPE_INVALID;
   GstHLSMedia *media;
   gchar *a, *v;
 
   media = g_new0 (GstHLSMedia, 1);
   media->ref_count = 1;
   media->playlist = gst_m3u8_new ();
+  media->mtype = GST_HLS_MEDIA_TYPE_INVALID;
 
   GST_LOG ("parsing %s", desc);
   while (desc != NULL && parse_attributes (&desc, &a, &v)) {
@@ -1114,7 +1406,7 @@ gst_m3u8_parse_media (gchar * desc, const gchar * base_uri)
   if (media->group_id == NULL || media->name == NULL)
     goto required_attributes_missing;
 
-  if (mtype == GST_HLS_MEDIA_TYPE_CLOSED_CAPTIONS)
+  if (media->mtype == GST_HLS_MEDIA_TYPE_CLOSED_CAPTIONS)
     goto uri_with_cc;
 
   GST_DEBUG ("media: %s, group '%s', name '%s', uri '%s', %s %s %s, lang=%s",
@@ -1229,6 +1521,8 @@ gst_hls_master_playlist_unref (GstHLSMasterPlaylist * playlist)
         (GDestroyNotify) gst_hls_variant_stream_unref);
     g_list_free_full (playlist->iframe_variants,
         (GDestroyNotify) gst_hls_variant_stream_unref);
+    if (playlist->default_variant)
+      gst_hls_variant_stream_unref (playlist->default_variant);
     g_free (playlist->last_data);
     g_free (playlist);
   }
@@ -1340,8 +1634,15 @@ gst_hls_master_playlist_new_from_data (gchar * data, const gchar * base_uri)
       data += stream->iframe ? 26 : 18;
       while (data && parse_attributes (&data, &a, &v)) {
         if (g_str_equal (a, "BANDWIDTH")) {
+          if (!stream->bandwidth) {
+            if (!int_from_string (v, NULL, &stream->bandwidth))
+              GST_WARNING ("Error while reading BANDWIDTH");
+          }
+        } else if (g_str_equal (a, "AVERAGE-BANDWIDTH")) {
+          GST_DEBUG
+              ("AVERAGE-BANDWIDTH attribute available. Using it as stream bandwidth");
           if (!int_from_string (v, NULL, &stream->bandwidth))
-            GST_WARNING ("Error while reading BANDWIDTH");
+            GST_WARNING ("Error while reading AVERAGE-BANDWIDTH");
         } else if (g_str_equal (a, "PROGRAM-ID")) {
           if (!int_from_string (v, NULL, &stream->program_id))
             GST_WARNING ("Error while reading PROGRAM-ID");
