@@ -59,9 +59,6 @@ struct _GstVulkanH264Decoder
 
   GstVulkanDecoder *decoder;
 
-  GstBuffer *inbuf;
-  GstMapInfo in_mapinfo;
-
   gboolean need_negotiation;
   gboolean need_params_update;
 
@@ -75,9 +72,6 @@ struct _GstVulkanH264Decoder
   VkChromaLocation xloc, yloc;
 
   GstVideoCodecState *output_state;
-
-  GstBufferPool *dpb_pool;
-  GstBuffer *layered_dpb;
 };
 
 static GstStaticPadTemplate gst_vulkan_h264dec_sink_template =
@@ -99,7 +93,7 @@ GST_DEBUG_CATEGORY (gst_debug_vulkan_h264_decoder);
 G_DEFINE_TYPE_WITH_CODE (GstVulkanH264Decoder, gst_vulkan_h264_decoder,
     GST_TYPE_H264_DECODER,
     GST_DEBUG_CATEGORY_INIT (gst_debug_vulkan_h264_decoder,
-        "vulkanh264dec", 0, "Vulkan H.H264 Decoder"));
+        "vulkanh264dec", 0, "Vulkan H.264 Decoder"));
 GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (vulkanh264dec, "vulkanh264dec",
     GST_RANK_NONE, GST_TYPE_VULKAN_H264_DECODER, vulkan_element_init (plugin));
 
@@ -111,7 +105,6 @@ _find_queues (GstVulkanDevice * device, GstVulkanQueue * queue, gpointer data)
       device->physical_device->queue_family_props[queue->family].queueFlags;
   guint32 codec =
       device->physical_device->queue_family_ops[queue->family].video;
-  gboolean ret;
 
   if (!self->graphic_queue
       && ((flags & VK_QUEUE_GRAPHICS_BIT) == VK_QUEUE_GRAPHICS_BIT)) {
@@ -126,13 +119,7 @@ _find_queues (GstVulkanDevice * device, GstVulkanQueue * queue, gpointer data)
     self->decode_queue = gst_object_ref (queue);
   }
 
-  ret = self->decode_queue && self->graphic_queue;
-
-  /* if graphic and decoder queues are the same, just keep one */
-  if (ret && (self->decode_queue == self->graphic_queue))
-    gst_clear_object (&self->graphic_queue);
-
-  return !ret;
+  return !(self->decode_queue && self->graphic_queue);
 }
 
 static gboolean
@@ -190,28 +177,25 @@ gst_vulkan_h264_decoder_close (GstVideoDecoder * decoder)
 {
   GstVulkanH264Decoder *self = GST_VULKAN_H264_DECODER (decoder);
 
-  if (self->decoder)
-    gst_vulkan_decoder_stop (self->decoder);
-
-  if (self->inbuf)
-    gst_buffer_unmap (self->inbuf, &self->in_mapinfo);
-  gst_clear_buffer (&self->inbuf);
-
-  if (self->output_state)
-    gst_video_codec_state_unref (self->output_state);
-
   gst_clear_object (&self->decoder);
   gst_clear_object (&self->decode_queue);
   gst_clear_object (&self->graphic_queue);
   gst_clear_object (&self->device);
   gst_clear_object (&self->instance);
 
-  if (self->dpb_pool) {
-    gst_buffer_pool_set_active (self->dpb_pool, FALSE);
-    gst_clear_object (&self->dpb_pool);
-  }
+  return TRUE;
+}
 
-  gst_clear_buffer (&self->layered_dpb);
+static gboolean
+gst_vulkan_h264_decoder_stop (GstVideoDecoder * decoder)
+{
+  GstVulkanH264Decoder *self = GST_VULKAN_H264_DECODER (decoder);
+
+  if (self->decoder)
+    gst_vulkan_decoder_stop (self->decoder);
+
+  if (self->output_state)
+    gst_video_codec_state_unref (self->output_state);
 
   return TRUE;
 }
@@ -1317,7 +1301,6 @@ gst_vulkan_h264_decoder_init (GstVulkanH264Decoder * self)
 static void
 gst_vulkan_h264_decoder_class_init (GstVulkanH264DecoderClass * klass)
 {
-  /* GObjectClass *gobject_class = G_OBJECT_CLASS (klass); */
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstVideoDecoderClass *decoder_class = GST_VIDEO_DECODER_CLASS (klass);
   GstH264DecoderClass *h264decoder_class = GST_H264_DECODER_CLASS (klass);
@@ -1337,6 +1320,7 @@ gst_vulkan_h264_decoder_class_init (GstVulkanH264DecoderClass * klass)
 
   decoder_class->open = GST_DEBUG_FUNCPTR (gst_vulkan_h264_decoder_open);
   decoder_class->close = GST_DEBUG_FUNCPTR (gst_vulkan_h264_decoder_close);
+  decoder_class->stop = GST_DEBUG_FUNCPTR (gst_vulkan_h264_decoder_stop);
   decoder_class->src_query =
       GST_DEBUG_FUNCPTR (gst_vulkan_h264_decoder_src_query);
   decoder_class->sink_query =
