@@ -24,13 +24,17 @@
 #include "transportstream.h"
 #include "transportsendbin.h"
 #include "transportreceivebin.h"
-#include "gstwebrtcice.h"
 #include "gstwebrtcbin.h"
 #include "utils.h"
 #include "gst/webrtc/webrtc-priv.h"
 
+#define GST_CAT_DEFAULT transport_stream_debug
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
+
 #define transport_stream_parent_class parent_class
-G_DEFINE_TYPE (TransportStream, transport_stream, GST_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_CODE (TransportStream, transport_stream, GST_TYPE_OBJECT,
+    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "webrtctransportstream", 0,
+        "webrtctransportstream"););
 
 enum
 {
@@ -59,7 +63,7 @@ transport_stream_get_pt (TransportStream * stream, const gchar * encoding_name,
     guint media_idx)
 {
   guint i;
-  gint ret = 0;
+  gint ret = -1;
 
   for (i = 0; i < stream->ptmap->len; i++) {
     PtMapItem *item = &g_array_index (stream->ptmap, PtMapItem, i);
@@ -165,25 +169,14 @@ transport_stream_dispose (GObject * object)
 {
   TransportStream *stream = TRANSPORT_STREAM (object);
 
-  if (stream->send_bin)
-    gst_object_unref (stream->send_bin);
-  stream->send_bin = NULL;
-
-  if (stream->receive_bin)
-    gst_object_unref (stream->receive_bin);
-  stream->receive_bin = NULL;
-
-  if (stream->transport)
-    gst_object_unref (stream->transport);
-  stream->transport = NULL;
-
-  if (stream->rtxsend)
-    gst_object_unref (stream->rtxsend);
-  stream->rtxsend = NULL;
-
-  if (stream->rtxreceive)
-    gst_object_unref (stream->rtxreceive);
-  stream->rtxreceive = NULL;
+  gst_clear_object (&stream->send_bin);
+  gst_clear_object (&stream->receive_bin);
+  gst_clear_object (&stream->transport);
+  gst_clear_object (&stream->rtxsend);
+  gst_clear_object (&stream->rtxreceive);
+  gst_clear_object (&stream->reddec);
+  g_list_free_full (stream->fecdecs, (GDestroyNotify) gst_object_unref);
+  stream->fecdecs = NULL;
 
   GST_OBJECT_PARENT (object) = NULL;
 
@@ -196,7 +189,12 @@ transport_stream_finalize (GObject * object)
   TransportStream *stream = TRANSPORT_STREAM (object);
 
   g_array_free (stream->ptmap, TRUE);
-  g_ptr_array_free (stream->remote_ssrcmap, TRUE);
+  g_ptr_array_free (stream->ssrcmap, TRUE);
+
+  gst_clear_object (&stream->rtxsend_stream_id);
+  gst_clear_object (&stream->rtxsend_repaired_stream_id);
+  gst_clear_object (&stream->rtxreceive_stream_id);
+  gst_clear_object (&stream->rtxreceive_repaired_stream_id);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -283,11 +281,13 @@ clear_ptmap_item (PtMapItem * item)
     gst_caps_unref (item->caps);
 }
 
-SsrcMapItem *
-ssrcmap_item_new (guint32 ssrc, guint media_idx)
+static SsrcMapItem *
+ssrcmap_item_new (GstWebRTCRTPTransceiverDirection direction, guint32 ssrc,
+    guint media_idx)
 {
-  SsrcMapItem *ssrc_item = g_slice_new (SsrcMapItem);
+  SsrcMapItem *ssrc_item = g_new0 (SsrcMapItem, 1);
 
+  ssrc_item->direction = direction;
   ssrc_item->media_idx = media_idx;
   ssrc_item->ssrc = ssrc;
   g_weak_ref_init (&ssrc_item->rtpjitterbuffer, NULL);
@@ -299,7 +299,66 @@ static void
 ssrcmap_item_free (SsrcMapItem * item)
 {
   g_weak_ref_clear (&item->rtpjitterbuffer);
-  g_slice_free (SsrcMapItem, item);
+  g_clear_pointer (&item->mid, g_free);
+  g_clear_pointer (&item->rid, g_free);
+  g_free (item);
+}
+
+SsrcMapItem *
+transport_stream_find_ssrc_map_item (TransportStream * stream,
+    gconstpointer data, FindSsrcMapFunc func)
+{
+  int i;
+
+  for (i = 0; i < stream->ssrcmap->len; i++) {
+    SsrcMapItem *item = g_ptr_array_index (stream->ssrcmap, i);
+
+    if (func (item, data))
+      return item;
+  }
+
+  return NULL;
+}
+
+void
+transport_stream_filter_ssrc_map_item (TransportStream * stream,
+    gconstpointer data, FindSsrcMapFunc func)
+{
+  int i;
+
+  for (i = 0; i < stream->ssrcmap->len;) {
+    SsrcMapItem *item = g_ptr_array_index (stream->ssrcmap, i);
+
+    if (!func (item, data)) {
+      GST_TRACE_OBJECT (stream, "removing ssrc %u", item->ssrc);
+      g_ptr_array_remove_index_fast (stream->ssrcmap, i);
+    } else {
+      i++;
+    }
+  }
+}
+
+SsrcMapItem *
+transport_stream_add_ssrc_map_item (TransportStream * stream,
+    GstWebRTCRTPTransceiverDirection direction, guint32 ssrc, guint media_idx)
+{
+  SsrcMapItem *ret = NULL;
+
+  g_return_val_if_fail (direction ==
+      GST_WEBRTC_RTP_TRANSCEIVER_DIRECTION_RECVONLY
+      || direction == GST_WEBRTC_RTP_TRANSCEIVER_DIRECTION_SENDONLY, NULL);
+  g_return_val_if_fail (ssrc != 0, NULL);
+
+  GST_INFO_OBJECT (stream, "Adding mapping for rtp session %u media_idx %u "
+      "direction %s ssrc %u", stream->session_id, media_idx,
+      gst_webrtc_rtp_transceiver_direction_to_string (direction), ssrc);
+
+  /* XXX: duplicates? */
+  ret = ssrcmap_item_new (direction, ssrc, media_idx);
+
+  g_ptr_array_add (stream->ssrcmap, ret);
+
+  return ret;
 }
 
 static void
@@ -307,8 +366,11 @@ transport_stream_init (TransportStream * stream)
 {
   stream->ptmap = g_array_new (FALSE, TRUE, sizeof (PtMapItem));
   g_array_set_clear_func (stream->ptmap, (GDestroyNotify) clear_ptmap_item);
-  stream->remote_ssrcmap = g_ptr_array_new_with_free_func (
+  stream->ssrcmap = g_ptr_array_new_with_free_func (
       (GDestroyNotify) ssrcmap_item_free);
+
+  stream->rtphdrext_id_stream_id = -1;
+  stream->rtphdrext_id_repaired_stream_id = -1;
 }
 
 TransportStream *
