@@ -23,15 +23,24 @@
 #endif
 
 #include "gstd3d11device.h"
+#include "gstd3d11device-private.h"
 #include "gstd3d11utils.h"
 #include "gstd3d11format.h"
 #include "gstd3d11-private.h"
 #include "gstd3d11memory.h"
+#include "gstd3d11compile.h"
+#include "gstd3d11shadercache.h"
 #include <gmodule.h>
 #include <wrl.h>
 
 #include <windows.h>
 #include <versionhelpers.h>
+#include <map>
+#include <utility>
+#include <atomic>
+#include <mutex>
+#include <string.h>
+#include <unordered_map>
 
 /**
  * SECTION:gstd3d11device
@@ -92,42 +101,53 @@ enum
 #define DEFAULT_ADAPTER 0
 #define DEFAULT_CREATE_FLAGS 0
 
+/* *INDENT-OFF* */
 struct _GstD3D11DevicePrivate
 {
-  guint adapter;
-  guint device_id;
-  guint vendor_id;
-  gboolean hardware;
-  gchar *description;
-  guint create_flags;
-  gint64 adapter_luid;
+  guint adapter = 0;
+  guint device_id = 0;
+  guint vendor_id = 0;
+  gboolean hardware = 0;
+  gchar *description = nullptr;
+  guint create_flags = 0;
+  gint64 adapter_luid = 0;
 
-  ID3D11Device *device;
-  ID3D11Device5 *device5;
-  ID3D11DeviceContext *device_context;
-  ID3D11DeviceContext4 *device_context4;
+  ID3D11Device *device = nullptr;
+  ID3D11Device5 *device5 = nullptr;
+  ID3D11DeviceContext *device_context = nullptr;
+  ID3D11DeviceContext4 *device_context4 = nullptr;
 
-  ID3D11VideoDevice *video_device;
-  ID3D11VideoContext *video_context;
+  ID3D11VideoDevice *video_device = nullptr;
+  ID3D11VideoContext *video_context = nullptr;
 
-  IDXGIFactory1 *factory;
-  GArray *format_table;
+  IDXGIFactory1 *factory = nullptr;
+  std::unordered_map<GstVideoFormat, GstD3D11Format> format_table;
 
-  CRITICAL_SECTION extern_lock;
-  SRWLOCK resource_lock;
+  std::recursive_mutex extern_lock;
+  std::mutex resource_lock;
 
   LARGE_INTEGER frequency;
 
+  D3D_FEATURE_LEVEL feature_level;
+  std::map <gint64, ComPtr<ID3D11PixelShader>> ps_cache;
+  std::map <gint64,
+      std::pair<ComPtr<ID3D11VertexShader>, ComPtr<ID3D11InputLayout>>> vs_cache;
+  std::map <D3D11_FILTER, ComPtr<ID3D11SamplerState>> sampler_cache;
+
+  ID3D11RasterizerState *rs = nullptr;
+  ID3D11RasterizerState *rs_msaa = nullptr;
+
 #if HAVE_D3D11SDKLAYERS_H
-  ID3D11Debug *d3d11_debug;
-  ID3D11InfoQueue *d3d11_info_queue;
+  ID3D11Debug *d3d11_debug = nullptr;
+  ID3D11InfoQueue *d3d11_info_queue = nullptr;
 #endif
 
 #if HAVE_DXGIDEBUG_H
-  IDXGIDebug *dxgi_debug;
-  IDXGIInfoQueue *dxgi_info_queue;
+  IDXGIDebug *dxgi_debug = nullptr;
+  IDXGIInfoQueue *dxgi_info_queue = nullptr;
 #endif
 };
+/* *INDENT-ON* */
 
 static void
 debug_init_once (void)
@@ -144,7 +164,7 @@ debug_init_once (void)
 
 #define gst_d3d11_device_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstD3D11Device, gst_d3d11_device, GST_TYPE_OBJECT,
-    G_ADD_PRIVATE (GstD3D11Device); debug_init_once ());
+    debug_init_once ());
 
 static void gst_d3d11_device_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
@@ -225,7 +245,7 @@ gst_d3d11_device_d3d11_debug (GstD3D11Device * device,
       return;
     }
 
-    msg = (D3D11_MESSAGE *) g_alloca (msg_len);
+    msg = (D3D11_MESSAGE *) g_malloc0 (msg_len);
     hr = info_queue->GetMessage (i, msg, &msg_len);
 
     level = d3d11_message_severity_to_gst (msg->Severity);
@@ -238,6 +258,7 @@ gst_d3d11_device_d3d11_debug (GstD3D11Device * device,
 
     gst_debug_log (gst_d3d11_debug_layer_debug, level, file, function, line,
         G_OBJECT (device), "D3D11InfoQueue: %s", msg->pDescription);
+    g_free (msg);
   }
 
   info_queue->ClearStoredMessages ();
@@ -336,12 +357,13 @@ gst_d3d11_device_dxgi_debug (GstD3D11Device * device,
       return;
     }
 
-    msg = (DXGI_INFO_QUEUE_MESSAGE *) g_alloca (msg_len);
+    msg = (DXGI_INFO_QUEUE_MESSAGE *) g_malloc0 (msg_len);
     hr = info_queue->GetMessage (DXGI_DEBUG_ALL, i, msg, &msg_len);
 
     level = dxgi_info_queue_message_severity_to_gst (msg->Severity);
     gst_debug_log (gst_d3d11_debug_layer_debug, level, file, function, line,
         G_OBJECT (device), "DXGIInfoQueue: %s", msg->pDescription);
+    g_free (msg);
   }
 
   info_queue->ClearStoredMessages (DXGI_DEBUG_ALL);
@@ -401,17 +423,7 @@ gst_d3d11_device_class_init (GstD3D11DeviceClass * klass)
 static void
 gst_d3d11_device_init (GstD3D11Device * self)
 {
-  GstD3D11DevicePrivate *priv;
-
-  priv = (GstD3D11DevicePrivate *)
-      gst_d3d11_device_get_instance_private (self);
-  priv->adapter = DEFAULT_ADAPTER;
-  priv->format_table = g_array_sized_new (FALSE, FALSE,
-      sizeof (GstD3D11Format), GST_D3D11_N_FORMATS);
-
-  InitializeCriticalSection (&priv->extern_lock);
-
-  self->priv = priv;
+  self->priv = new GstD3D11DevicePrivate ();
 }
 
 static gboolean
@@ -509,6 +521,12 @@ gst_d3d11_device_setup_format_table (GstD3D11Device * self)
       case GST_VIDEO_FORMAT_P012_LE:
       case GST_VIDEO_FORMAT_P016_LE:
       case GST_VIDEO_FORMAT_YUY2:
+      case GST_VIDEO_FORMAT_Y210:
+      case GST_VIDEO_FORMAT_Y212_LE:
+      case GST_VIDEO_FORMAT_Y412_LE:
+      case GST_VIDEO_FORMAT_BGRA64_LE:
+      case GST_VIDEO_FORMAT_BGR10A2_LE:
+      case GST_VIDEO_FORMAT_RBGA:
       {
         gboolean supported = TRUE;
 
@@ -558,12 +576,31 @@ gst_d3d11_device_setup_format_table (GstD3D11Device * self)
       case GST_VIDEO_FORMAT_Y444_16LE:
       case GST_VIDEO_FORMAT_AYUV:
       case GST_VIDEO_FORMAT_AYUV64:
+      case GST_VIDEO_FORMAT_UYVY:
+      case GST_VIDEO_FORMAT_VYUY:
+      case GST_VIDEO_FORMAT_YVYU:
+      case GST_VIDEO_FORMAT_ARGB:
+      case GST_VIDEO_FORMAT_xRGB:
+      case GST_VIDEO_FORMAT_ABGR:
+      case GST_VIDEO_FORMAT_xBGR:
+      case GST_VIDEO_FORMAT_RGB:
+      case GST_VIDEO_FORMAT_BGR:
+      case GST_VIDEO_FORMAT_v210:
+      case GST_VIDEO_FORMAT_v216:
+      case GST_VIDEO_FORMAT_v308:
+      case GST_VIDEO_FORMAT_IYU2:
+      case GST_VIDEO_FORMAT_RGB16:
+      case GST_VIDEO_FORMAT_BGR16:
+      case GST_VIDEO_FORMAT_RGB15:
+      case GST_VIDEO_FORMAT_BGR15:
+      case GST_VIDEO_FORMAT_r210:
         /* RGB planar formats */
       case GST_VIDEO_FORMAT_RGBP:
       case GST_VIDEO_FORMAT_BGRP:
       case GST_VIDEO_FORMAT_GBR:
       case GST_VIDEO_FORMAT_GBR_10LE:
       case GST_VIDEO_FORMAT_GBR_12LE:
+      case GST_VIDEO_FORMAT_GBR_16LE:
       case GST_VIDEO_FORMAT_GBRA:
       case GST_VIDEO_FORMAT_GBRA_10LE:
       case GST_VIDEO_FORMAT_GBRA_12LE:
@@ -602,66 +639,13 @@ gst_d3d11_device_setup_format_table (GstD3D11Device * self)
     for (guint j = 0; j < GST_VIDEO_MAX_PLANES; j++)
       format.format_support[j] = support[j];
 
+#ifndef GST_DISABLE_GST_DEBUG
     if (gst_debug_category_get_threshold (GST_CAT_DEFAULT) >= GST_LEVEL_LOG)
       dump_format (self, &format);
-
-    g_array_append_val (priv->format_table, format);
-  }
-
-  /* FIXME: d3d11 sampler doesn't support packed-and-subsampled formats
-   * very well (and it's really poorly documented).
-   * As per observation, d3d11 samplers seems to be dropping the second
-   * Y componet from "Y0-U0-Y1-V0" pair which results in bad visual quality
-   * than 4:2:0 subsampled formats. We should revisit this later */
-
-  /* TODO: The best would be using d3d11 compute shader to handle this kinds of
-   * samples but comute shader is not implemented yet by us.
-   *
-   * Another simple approach is using d3d11 video processor,
-   * but capability will be very device dependent because it depends on
-   * GPU vendor's driver implementation, moreover, software fallback does
-   * not support d3d11 video processor. So it's not reliable in this case */
-#if 0
-  /* NOTE: packted yuv 4:2:2 YUY2, UYVY, and VYUY formats are not natively
-   * supported render target view formats
-   * (i.e., cannot be output format of shader pipeline) */
-  priv->format_table[n_formats].format = GST_VIDEO_FORMAT_YUY2;
-  if (can_support_format (self, DXGI_FORMAT_YUY2,
-          D3D11_FORMAT_SUPPORT_SHADER_SAMPLE)) {
-    priv->format_table[n_formats].resource_format[0] =
-        DXGI_FORMAT_R8G8B8A8_UNORM;
-    priv->format_table[n_formats].dxgi_format = DXGI_FORMAT_YUY2;
-  } else {
-    /* If DXGI_FORMAT_YUY2 format is not supported, use this format,
-     * it's analogous to YUY2 */
-    priv->format_table[n_formats].resource_format[0] =
-        DXGI_FORMAT_G8R8_G8B8_UNORM;
-  }
-  n_formats++;
-
-  /* No native DXGI format available for UYVY */
-  priv->format_table[n_formats].format = GST_VIDEO_FORMAT_UYVY;
-  priv->format_table[n_formats].resource_format[0] =
-      DXGI_FORMAT_R8G8_B8G8_UNORM;
-  n_formats++;
-
-  /* No native DXGI format available for VYUY */
-  priv->format_table[n_formats].format = GST_VIDEO_FORMAT_VYUY;
-  priv->format_table[n_formats].resource_format[0] =
-      DXGI_FORMAT_R8G8_B8G8_UNORM;
-  n_formats++;
-
-  /* Y210 and Y410 formats cannot support rtv */
-  priv->format_table[n_formats].format = GST_VIDEO_FORMAT_Y210;
-  priv->format_table[n_formats].resource_format[0] =
-      DXGI_FORMAT_R16G16B16A16_UNORM;
-  if (can_support_format (self, DXGI_FORMAT_Y210,
-          D3D11_FORMAT_SUPPORT_SHADER_SAMPLE))
-    priv->format_table[n_formats].dxgi_format = DXGI_FORMAT_Y210;
-  else
-    priv->format_table[n_formats].dxgi_format = DXGI_FORMAT_UNKNOWN;
-  n_formats++;
 #endif
+
+    priv->format_table[format.format] = format;
+  }
 }
 
 static void
@@ -696,6 +680,31 @@ gst_d3d11_device_get_property (GObject * object, guint prop_id,
   }
 }
 
+void
+gst_d3d11_device_log_live_objects (GstD3D11Device * device,
+    const gchar * file, const gchar * function, gint line)
+{
+#if HAVE_D3D11SDKLAYERS_H
+  if (device->priv->d3d11_debug) {
+    device->priv->d3d11_debug->ReportLiveDeviceObjects ((D3D11_RLDO_FLAGS)
+        GST_D3D11_RLDO_FLAGS);
+  }
+
+  if (device->priv->d3d11_info_queue)
+    gst_d3d11_device_d3d11_debug (device, file, function, line);
+#endif
+
+#if HAVE_DXGIDEBUG_H
+  if (device->priv->dxgi_debug) {
+    device->priv->dxgi_debug->ReportLiveObjects (DXGI_DEBUG_ALL,
+        (DXGI_DEBUG_RLO_FLAGS) GST_D3D11_RLDO_FLAGS);
+  }
+
+  if (device->priv->dxgi_info_queue)
+    gst_d3d11_device_dxgi_debug (device, file, function, line);
+#endif
+}
+
 static void
 gst_d3d11_device_dispose (GObject * object)
 {
@@ -704,6 +713,12 @@ gst_d3d11_device_dispose (GObject * object)
 
   GST_LOG_OBJECT (self, "dispose");
 
+  priv->ps_cache.clear ();
+  priv->vs_cache.clear ();
+  priv->sampler_cache.clear ();
+
+  GST_D3D11_CLEAR_COM (priv->rs);
+  GST_D3D11_CLEAR_COM (priv->rs_msaa);
   GST_D3D11_CLEAR_COM (priv->device5);
   GST_D3D11_CLEAR_COM (priv->device_context4);
   GST_D3D11_CLEAR_COM (priv->video_device);
@@ -711,29 +726,15 @@ gst_d3d11_device_dispose (GObject * object)
   GST_D3D11_CLEAR_COM (priv->device);
   GST_D3D11_CLEAR_COM (priv->device_context);
   GST_D3D11_CLEAR_COM (priv->factory);
+  gst_d3d11_device_log_live_objects (self, __FILE__, GST_FUNCTION, __LINE__);
+
 #if HAVE_D3D11SDKLAYERS_H
-  if (priv->d3d11_debug) {
-    priv->d3d11_debug->ReportLiveDeviceObjects ((D3D11_RLDO_FLAGS)
-        GST_D3D11_RLDO_FLAGS);
-  }
   GST_D3D11_CLEAR_COM (priv->d3d11_debug);
-
-  if (priv->d3d11_info_queue)
-    gst_d3d11_device_d3d11_debug (self, __FILE__, GST_FUNCTION, __LINE__);
-
   GST_D3D11_CLEAR_COM (priv->d3d11_info_queue);
 #endif
 
 #if HAVE_DXGIDEBUG_H
-  if (priv->dxgi_debug) {
-    priv->dxgi_debug->ReportLiveObjects (DXGI_DEBUG_ALL,
-        (DXGI_DEBUG_RLO_FLAGS) GST_D3D11_RLDO_FLAGS);
-  }
   GST_D3D11_CLEAR_COM (priv->dxgi_debug);
-
-  if (priv->dxgi_info_queue)
-    gst_d3d11_device_dxgi_debug (self, __FILE__, GST_FUNCTION, __LINE__);
-
   GST_D3D11_CLEAR_COM (priv->dxgi_info_queue);
 #endif
 
@@ -748,10 +749,10 @@ gst_d3d11_device_finalize (GObject * object)
 
   GST_LOG_OBJECT (self, "finalize");
 
-  g_array_unref (priv->format_table);
-  DeleteCriticalSection (&priv->extern_lock);
   g_free (priv->description);
 
+  delete priv;
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -833,7 +834,6 @@ _gst_d3d11_device_get_adapter (const GstD3D11DeviceConstructData * data,
       ComPtr < IDXGIDevice > dxgi_device;
       ComPtr < IDXGIAdapter > adapter;
       ID3D11Device *device = data->data.device;
-      guint luid;
 
       hr = device->QueryInterface (IID_PPV_ARGS (&dxgi_device));
       if (FAILED (hr))
@@ -851,7 +851,7 @@ _gst_d3d11_device_get_adapter (const GstD3D11DeviceConstructData * data,
       if (FAILED (hr))
         return hr;
 
-      luid = gst_d3d11_luid_to_int64 (&desc.AdapterLuid);
+      auto luid = gst_d3d11_luid_to_int64 (&desc.AdapterLuid);
 
       for (guint i = 0;; i++) {
         DXGI_ADAPTER_DESC tmp_desc;
@@ -965,9 +965,6 @@ gst_d3d11_device_new_internal (const GstD3D11DeviceConstructData * data)
     D3D_FEATURE_LEVEL_11_0,
     D3D_FEATURE_LEVEL_10_1,
     D3D_FEATURE_LEVEL_10_0,
-    D3D_FEATURE_LEVEL_9_3,
-    D3D_FEATURE_LEVEL_9_2,
-    D3D_FEATURE_LEVEL_9_1
   };
   D3D_FEATURE_LEVEL selected_level;
 
@@ -1008,6 +1005,12 @@ gst_d3d11_device_new_internal (const GstD3D11DeviceConstructData * data)
       return nullptr;
     }
 
+    selected_level = device->GetFeatureLevel ();
+    if (selected_level < D3D_FEATURE_LEVEL_10_0) {
+      GST_ERROR ("Feature level 0x%x is not supported", (guint) selected_level);
+      return nullptr;
+    }
+
     device->GetImmediateContext (&device_context);
   }
 
@@ -1099,6 +1102,7 @@ gst_d3d11_device_new_internal (const GstD3D11DeviceConstructData * data)
   priv->description = g_utf16_to_utf8 ((gunichar2 *) adapter_desc.Description,
       -1, nullptr, nullptr, nullptr);
   priv->adapter_luid = gst_d3d11_luid_to_int64 (&adapter_desc.AdapterLuid);
+  priv->feature_level = priv->device->GetFeatureLevel ();
 
   DXGI_ADAPTER_DESC1 desc1;
   hr = adapter->GetDesc1 (&desc1);
@@ -1262,7 +1266,7 @@ gst_d3d11_device_get_video_device_handle (GstD3D11Device * device)
   g_return_val_if_fail (GST_IS_D3D11_DEVICE (device), NULL);
 
   priv = device->priv;
-  GstD3D11SRWLockGuard lk (&priv->resource_lock);
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
   if (!priv->video_device) {
     HRESULT hr;
     ID3D11VideoDevice *video_device = NULL;
@@ -1295,7 +1299,7 @@ gst_d3d11_device_get_video_context_handle (GstD3D11Device * device)
   g_return_val_if_fail (GST_IS_D3D11_DEVICE (device), NULL);
 
   priv = device->priv;
-  GstD3D11SRWLockGuard lk (&priv->resource_lock);
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
   if (!priv->video_context) {
     HRESULT hr;
     ID3D11VideoContext *video_context = NULL;
@@ -1328,7 +1332,7 @@ gst_d3d11_device_lock (GstD3D11Device * device)
   priv = device->priv;
 
   GST_TRACE_OBJECT (device, "device locking");
-  EnterCriticalSection (&priv->extern_lock);
+  priv->extern_lock.lock ();
   GST_TRACE_OBJECT (device, "device locked");
 }
 
@@ -1350,7 +1354,7 @@ gst_d3d11_device_unlock (GstD3D11Device * device)
 
   priv = device->priv;
 
-  LeaveCriticalSection (&priv->extern_lock);
+  priv->extern_lock.unlock ();
   GST_TRACE_OBJECT (device, "device unlocked");
 }
 
@@ -1377,23 +1381,18 @@ gst_d3d11_device_get_format (GstD3D11Device * device, GstVideoFormat format,
 
   priv = device->priv;
 
-  for (guint i = 0; i < priv->format_table->len; i++) {
-    const GstD3D11Format *d3d11_fmt =
-        &g_array_index (priv->format_table, GstD3D11Format, i);
-
-    if (d3d11_fmt->format != format)
-      continue;
-
+  const auto & target = priv->format_table.find (format);
+  if (target == priv->format_table.end ()) {
     if (device_format)
-      *device_format = *d3d11_fmt;
+      gst_d3d11_format_init (device_format);
 
-    return TRUE;
+    return FALSE;
   }
 
   if (device_format)
-    gst_d3d11_format_init (device_format);
+    *device_format = target->second;
 
-  return FALSE;
+  return TRUE;
 }
 
 GST_DEFINE_MINI_OBJECT_TYPE (GstD3D11Fence, gst_d3d11_fence);
@@ -1650,3 +1649,286 @@ gst_d3d11_fence_wait (GstD3D11Fence * fence)
 
   return TRUE;
 }
+
+gint64
+gst_d3d11_pixel_shader_token_new (void)
+{
+  /* *INDENT-OFF* */
+  static std::atomic < gint64 > token_ { 0 };
+  /* *INDENT-ON* */
+
+  return token_.fetch_add (1);
+}
+
+gint64
+gst_d3d11_vertex_shader_token_new (void)
+{
+  /* *INDENT-OFF* */
+  static std::atomic < gint64 > token_ { 0 };
+  /* *INDENT-ON* */
+
+  return token_.fetch_add (1);
+}
+
+gint64
+gst_d3d11_compute_shader_token_new (void)
+{
+  /* *INDENT-OFF* */
+  static std::atomic < gint64 > token_ { 0 };
+  /* *INDENT-ON* */
+
+  return token_.fetch_add (1);
+}
+
+HRESULT
+gst_d3d11_device_get_pixel_shader_uncached (GstD3D11Device * device,
+    gint64 token, const void *bytecode, gsize bytecode_size,
+    const gchar * source, gsize source_size, const gchar * entry_point,
+    const D3D_SHADER_MACRO * defines, ID3D11PixelShader ** ps)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  HRESULT hr;
+  ComPtr < ID3D11PixelShader > shader;
+  ComPtr < ID3DBlob > blob;
+  const void *data;
+  gsize size;
+
+  GST_LOG_OBJECT (device,
+      "Creating pixel shader for token %" G_GINT64_FORMAT ", source:\n%s",
+      token, source);
+
+  if (bytecode && bytecode_size > 1) {
+    data = bytecode;
+    size = bytecode_size;
+    GST_DEBUG_OBJECT (device,
+        "Creating shader \"%s\" using precompiled bytecode", entry_point);
+  } else {
+    hr = gst_d3d11_shader_cache_get_pixel_shader_blob (token,
+        source, source_size, entry_point, defines, &blob);
+    if (!gst_d3d11_result (hr, device))
+      return hr;
+
+    data = blob->GetBufferPointer ();
+    size = blob->GetBufferSize ();
+  }
+
+  hr = priv->device->CreatePixelShader (data, size, nullptr, &shader);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  GST_DEBUG_OBJECT (device,
+      "Created pixel shader \"%s\" for token %" G_GINT64_FORMAT,
+      entry_point, token);
+  *ps = shader.Detach ();
+
+  return S_OK;
+}
+
+HRESULT
+gst_d3d11_device_get_pixel_shader (GstD3D11Device * device, gint64 token,
+    const void *bytecode, gsize bytecode_size, const gchar * source,
+    gsize source_size, const gchar * entry_point,
+    const D3D_SHADER_MACRO * defines, ID3D11PixelShader ** ps)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  HRESULT hr;
+  ComPtr < ID3D11PixelShader > shader;
+
+  GST_DEBUG_OBJECT (device, "Getting pixel shader \"%s\" for token %"
+      G_GINT64_FORMAT, entry_point, token);
+
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
+  auto cached = priv->ps_cache.find (token);
+  if (cached != priv->ps_cache.end ()) {
+    GST_DEBUG_OBJECT (device,
+        "Found cached pixel shader \"%s\" for token %" G_GINT64_FORMAT,
+        entry_point, token);
+    *ps = cached->second.Get ();
+    (*ps)->AddRef ();
+    return S_OK;
+  }
+
+  hr = gst_d3d11_device_get_pixel_shader_uncached (device, token, bytecode,
+      bytecode_size, source, source_size, entry_point, defines, &shader);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  priv->ps_cache[token] = shader;
+  *ps = shader.Detach ();
+
+  return S_OK;
+}
+
+HRESULT
+gst_d3d11_device_get_vertex_shader (GstD3D11Device * device, gint64 token,
+    const void *bytecode, gsize bytecode_size, const gchar * source,
+    gsize source_size, const gchar * entry_point,
+    const D3D11_INPUT_ELEMENT_DESC * input_desc, guint desc_len,
+    ID3D11VertexShader ** vs, ID3D11InputLayout ** layout)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  HRESULT hr;
+  ComPtr < ID3D11VertexShader > shader;
+  ComPtr < ID3D11InputLayout > input_layout;
+  ComPtr < ID3DBlob > blob;
+  const void *data;
+  gsize size;
+
+  GST_DEBUG_OBJECT (device, "Getting vertext shader \"%s\" for token %"
+      G_GINT64_FORMAT, entry_point, token);
+
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
+  auto cached = priv->vs_cache.find (token);
+  if (cached != priv->vs_cache.end ()) {
+    GST_DEBUG_OBJECT (device,
+        "Found cached vertex shader \"%s\" for token %" G_GINT64_FORMAT,
+        entry_point, token);
+    *vs = cached->second.first.Get ();
+    *layout = cached->second.second.Get ();
+    (*vs)->AddRef ();
+    (*layout)->AddRef ();
+    return S_OK;
+  }
+
+  GST_LOG_OBJECT (device,
+      "Creating vertex shader for token %" G_GINT64_FORMAT ", shader: \n%s",
+      token, source);
+
+  if (bytecode && bytecode_size > 1) {
+    data = bytecode;
+    size = bytecode_size;
+    GST_DEBUG_OBJECT (device,
+        "Creating shader \"%s\" using precompiled bytecode", entry_point);
+  } else {
+    hr = gst_d3d11_shader_cache_get_vertex_shader_blob (token,
+        source, source_size, entry_point, &blob);
+    if (!gst_d3d11_result (hr, device))
+      return hr;
+
+    data = blob->GetBufferPointer ();
+    size = blob->GetBufferSize ();
+  }
+
+  hr = priv->device->CreateVertexShader (data, size, nullptr, &shader);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  hr = priv->device->CreateInputLayout (input_desc, desc_len, data,
+      size, &input_layout);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  GST_DEBUG_OBJECT (device, "Created vertex shader \"%s\" for token %"
+      G_GINT64_FORMAT, entry_point, token);
+  priv->vs_cache[token] = std::make_pair (shader, input_layout);
+
+  *vs = shader.Detach ();
+  *layout = input_layout.Detach ();
+
+  return S_OK;
+}
+
+HRESULT
+gst_d3d11_device_get_sampler (GstD3D11Device * device, D3D11_FILTER filter,
+    ID3D11SamplerState ** sampler)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  ComPtr < ID3D11SamplerState > state;
+  D3D11_SAMPLER_DESC desc;
+  HRESULT hr;
+
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
+  auto cached = priv->sampler_cache.find (filter);
+  if (cached != priv->sampler_cache.end ()) {
+    *sampler = cached->second.Get ();
+    (*sampler)->AddRef ();
+    return S_OK;
+  }
+
+  memset (&desc, 0, sizeof (D3D11_SAMPLER_DESC));
+
+  desc.Filter = filter;
+  desc.AddressU = D3D11_TEXTURE_ADDRESS_CLAMP;
+  desc.AddressV = D3D11_TEXTURE_ADDRESS_CLAMP;
+  desc.AddressW = D3D11_TEXTURE_ADDRESS_CLAMP;
+  desc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
+  desc.MaxLOD = D3D11_FLOAT32_MAX;
+
+  if (filter == D3D11_FILTER_ANISOTROPIC) {
+    if (priv->feature_level > D3D_FEATURE_LEVEL_9_1)
+      desc.MaxAnisotropy = 16;
+    else
+      desc.MaxAnisotropy = 2;
+  }
+
+  hr = priv->device->CreateSamplerState (&desc, &state);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  priv->sampler_cache[filter] = state;
+  *sampler = state.Detach ();
+
+  return S_OK;
+}
+
+HRESULT
+gst_d3d11_device_get_rasterizer (GstD3D11Device * device,
+    ID3D11RasterizerState ** rasterizer)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  D3D11_RASTERIZER_DESC desc;
+  HRESULT hr;
+
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
+  if (priv->rs) {
+    *rasterizer = priv->rs;
+    priv->rs->AddRef ();
+    return S_OK;
+  }
+
+  memset (&desc, 0, sizeof (D3D11_RASTERIZER_DESC));
+  desc.FillMode = D3D11_FILL_SOLID;
+  desc.CullMode = D3D11_CULL_NONE;
+  desc.DepthClipEnable = TRUE;
+
+  hr = priv->device->CreateRasterizerState (&desc, rasterizer);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  priv->rs = *rasterizer;
+  priv->rs->AddRef ();
+
+  return S_OK;
+}
+
+HRESULT
+gst_d3d11_device_get_rasterizer_msaa (GstD3D11Device * device,
+    ID3D11RasterizerState ** rasterizer)
+{
+  GstD3D11DevicePrivate *priv = device->priv;
+  D3D11_RASTERIZER_DESC desc;
+  HRESULT hr;
+
+  std::lock_guard < std::mutex > lk (priv->resource_lock);
+  if (priv->rs_msaa) {
+    *rasterizer = priv->rs_msaa;
+    priv->rs_msaa->AddRef ();
+    return S_OK;
+  }
+
+  memset (&desc, 0, sizeof (D3D11_RASTERIZER_DESC));
+  desc.FillMode = D3D11_FILL_SOLID;
+  desc.CullMode = D3D11_CULL_NONE;
+  desc.DepthClipEnable = TRUE;
+  desc.MultisampleEnable = TRUE;
+  desc.AntialiasedLineEnable = TRUE;
+
+  hr = priv->device->CreateRasterizerState (&desc, rasterizer);
+  if (!gst_d3d11_result (hr, device))
+    return hr;
+
+  priv->rs_msaa = *rasterizer;
+  priv->rs_msaa->AddRef ();
+
+  return S_OK;
+}
