@@ -57,6 +57,8 @@ gst_rtp_asf_pay_set_caps (GstRTPBasePayload * rtppay, GstCaps * caps);
 
 #define gst_rtp_asf_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpAsfPay, gst_rtp_asf_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE (rtpasfpay, "rtpasfpay",
+    GST_RANK_NONE, GST_TYPE_RTP_ASF_PAY);
 
 static void
 gst_rtp_asf_pay_init (GstRtpAsfPay * rtpasfpay)
@@ -96,10 +98,10 @@ gst_rtp_asf_pay_class_init (GstRtpAsfPayClass * klass)
   gstbasertppayload_class->handle_buffer = gst_rtp_asf_pay_handle_buffer;
   gstbasertppayload_class->set_caps = gst_rtp_asf_pay_set_caps;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_asf_pay_sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_asf_pay_src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_asf_pay_sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_asf_pay_src_template);
   gst_element_class_set_static_metadata (gstelement_class, "RTP ASF payloader",
       "Codec/Payloader/Network",
       "Payload-encodes ASF into RTP packets (MS_RTSP)",
@@ -460,10 +462,3 @@ gst_rtp_asf_pay_handle_buffer (GstRTPBasePayload * rtppay, GstBuffer * buffer)
   gst_buffer_unref (buffer);
   return GST_FLOW_OK;
 }
-
-gboolean
-gst_rtp_asf_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpasfpay",
-      GST_RANK_NONE, GST_TYPE_RTP_ASF_PAY);
-}
