@@ -67,7 +67,7 @@ static void psmux_stream_find_pts_dts_within (PsMuxStream * stream, guint bound,
 PsMuxStream *
 psmux_stream_new (PsMux * mux, PsMuxStreamType stream_type)
 {
-  PsMuxStream *stream = g_slice_new0 (PsMuxStream);
+  PsMuxStream *stream = g_new0 (PsMuxStream, 1);
   PsMuxStreamIdInfo *info = &(mux->id_info);
 
   stream->stream_type = stream_type;
@@ -147,7 +147,7 @@ psmux_stream_new (PsMux * mux, PsMuxStreamType stream_type)
   if (stream->stream_id == 0) {
     g_critical ("Number of elementary streams of type %04x exceeds maximum",
         stream->stream_type);
-    g_slice_free (PsMuxStream, stream);
+    g_free (stream);
     return NULL;
   }
 
@@ -211,7 +211,7 @@ psmux_stream_free (PsMuxStream * stream)
   if (psmux_stream_bytes_in_buffer (stream)) {
     g_warning ("Freeing stream with data not yet processed");
   }
-  g_slice_free (PsMuxStream, stream);
+  g_free (stream);
 }
 
 /* Advance the current packet stream position by len bytes.
@@ -237,7 +237,7 @@ psmux_stream_consume (PsMuxStream * stream, guint len)
 
     gst_buffer_unmap (stream->cur_buffer->buf, &stream->cur_buffer->map);
     gst_buffer_unref (stream->cur_buffer->buf);
-    g_slice_free (PsMuxStreamBuffer, stream->cur_buffer);
+    g_free (stream->cur_buffer);
     stream->cur_buffer = NULL;
   }
 }
@@ -283,7 +283,7 @@ psmux_stream_get_data (PsMuxStream * stream, guint8 * buf, guint len)
       MIN (psmux_stream_bytes_in_buffer (stream), len - PSMUX_PES_MAX_HDR_LEN);
   /* Note that we cannot make a better estimation of the header length for the
    * time being; because the header length is dependent on whether we can find a
-   * timestamp in the upcomming buffers, which in turn depends on
+   * timestamp in the upcoming buffers, which in turn depends on
    * cur_pes_payload_size, which is exactly what we want to decide.
    */
 
@@ -463,7 +463,7 @@ psmux_stream_write_pes_header (PsMuxStream * stream, guint8 * data)
     if (stream->pi.flags & PSMUX_PACKET_FLAG_PES_EXT_STREAMID) {
       guint8 ext_len;
 
-      flags = 0x0f;             /* preceeding flags all 0 | (reserved bits) | PES_extension_flag_2 */
+      flags = 0x0f;             /* preceding flags all 0 | (reserved bits) | PES_extension_flag_2 */
       *data++ = flags;
 
       ext_len = 1;              /* Only writing 1 byte into the extended fields */
@@ -494,13 +494,13 @@ psmux_stream_add_data (PsMuxStream * stream, GstBuffer * buffer,
 
   g_return_if_fail (stream != NULL);
 
-  packet = g_slice_new (PsMuxStreamBuffer);
+  packet = g_new (PsMuxStreamBuffer, 1);
   packet->buf = buffer;
 
   if (!gst_buffer_map (packet->buf, &packet->map, GST_MAP_READ)) {
     GST_ERROR ("Failed to map buffer for reading");
     gst_buffer_unref (packet->buf);
-    g_slice_free (PsMuxStreamBuffer, packet);
+    g_free (packet);
     return;
   }
 
