@@ -122,6 +122,8 @@ static void gst_rist_rtx_send_finalize (GObject * object);
 G_DEFINE_TYPE_WITH_CODE (GstRistRtxSend, gst_rist_rtx_send, GST_TYPE_ELEMENT,
     GST_DEBUG_CATEGORY_INIT (gst_rist_rtx_send_debug, "ristrtxsend", 0,
         "RIST retransmission sender"));
+GST_ELEMENT_REGISTER_DEFINE (ristrtxsend, "ristrtxsend", GST_RANK_NONE,
+    GST_TYPE_RIST_RTX_SEND);
 
 typedef struct
 {
@@ -134,7 +136,7 @@ static void
 buffer_queue_item_free (BufferQueueItem * item)
 {
   gst_buffer_unref (item->buffer);
-  g_slice_free (BufferQueueItem, item);
+  g_free (item);
 }
 
 typedef struct
@@ -155,7 +157,7 @@ typedef struct
 static SSRCRtxData *
 ssrc_rtx_data_new (guint32 rtx_ssrc)
 {
-  SSRCRtxData *data = g_slice_new0 (SSRCRtxData);
+  SSRCRtxData *data = g_new0 (SSRCRtxData, 1);
 
   data->rtx_ssrc = rtx_ssrc;
   data->next_seqnum = data->seqnum_base = g_random_int_range (0, G_MAXUINT16);
@@ -169,7 +171,7 @@ static void
 ssrc_rtx_data_free (SSRCRtxData * data)
 {
   g_sequence_free (data->queue);
-  g_slice_free (SSRCRtxData, data);
+  g_free (data);
 }
 
 static void
@@ -303,7 +305,7 @@ gst_rtp_rtx_data_queue_item_free (gpointer item)
   GstDataQueueItem *data = item;
   if (data->object)
     gst_mini_object_unref (data->object);
-  g_slice_free (GstDataQueueItem, data);
+  g_free (data);
 }
 
 static gboolean
@@ -312,7 +314,7 @@ gst_rist_rtx_send_push_out (GstRistRtxSend * rtx, gpointer object)
   GstDataQueueItem *data;
   gboolean success;
 
-  data = g_slice_new0 (GstDataQueueItem);
+  data = g_new0 (GstDataQueueItem, 1);
   data->object = GST_MINI_OBJECT (object);
   data->size = 1;
   data->duration = 1;
@@ -614,7 +616,7 @@ process_buffer (GstRistRtxSend * rtx, GstBuffer * buffer)
     extseqnum = gst_rist_rtp_ext_seq (&data->max_extseqnum, seqnum);
 
   /* add current rtp buffer to queue history */
-  item = g_slice_new0 (BufferQueueItem);
+  item = g_new0 (BufferQueueItem, 1);
   item->extseqnum = extseqnum;
   item->timestamp = rtptime;
   item->buffer = gst_buffer_ref (buffer);
