@@ -24,30 +24,23 @@
 #include <gst/gst.h>
 #include <gst/video/video.h>
 #include <gst/audio/audio.h>
-#include <jni.h>
-
-#include "gstjniutils.h"
+#include "gstamc-codec.h"
+#include "gstamc-codeclist.h"
+#include "gstamc-format.h"
 
 G_BEGIN_DECLS
 
-typedef struct _GstAmcCodecInfo GstAmcCodecInfo;
 typedef struct _GstAmcCodecType GstAmcCodecType;
-typedef struct _GstAmcCodec GstAmcCodec;
-typedef struct _GstAmcBufferInfo GstAmcBufferInfo;
-typedef struct _GstAmcFormat GstAmcFormat;
-typedef struct _GstAmcColorFormatInfo GstAmcColorFormatInfo;
+typedef struct _GstAmcCodecInfo GstAmcCodecInfo;
 
 struct _GstAmcCodecType {
   gchar *mime;
 
   gint *color_formats;
-  gint n_color_formats;
+  gsize n_color_formats;
 
-  struct {
-    gint profile;
-    gint level;
-  } *profile_levels;
-  gint n_profile_levels;
+  GstAmcCodecProfileLevel * profile_levels;
+  gsize n_profile_levels;
 };
 
 struct _GstAmcCodecInfo {
@@ -58,65 +51,9 @@ struct _GstAmcCodecInfo {
   gint n_supported_types;
 };
 
-struct _GstAmcFormat {
-  /* < private > */
-  jobject object; /* global reference */
-};
-
-struct _GstAmcCodec {
-  /* < private > */
-  jobject object; /* global reference */
-
-  GstAmcBuffer *input_buffers, *output_buffers;
-  gsize n_input_buffers, n_output_buffers;
-};
-
-struct _GstAmcBufferInfo {
-  gint flags;
-  gint offset;
-  gint64 presentation_time_us;
-  gint size;
-};
-
 extern GQuark gst_amc_codec_info_quark;
 
-GstAmcCodec * gst_amc_codec_new (const gchar *name, GError **err);
-void gst_amc_codec_free (GstAmcCodec * codec);
-
-gboolean gst_amc_codec_configure (GstAmcCodec * codec, GstAmcFormat * format, jobject surface, gint flags, GError **err);
-GstAmcFormat * gst_amc_codec_get_output_format (GstAmcCodec * codec, GError **err);
-
-gboolean gst_amc_codec_start (GstAmcCodec * codec, GError **err);
-gboolean gst_amc_codec_stop (GstAmcCodec * codec, GError **err);
-gboolean gst_amc_codec_flush (GstAmcCodec * codec, GError **err);
-gboolean gst_amc_codec_release (GstAmcCodec * codec, GError **err);
-
-GstAmcBuffer * gst_amc_codec_get_output_buffer (GstAmcCodec * codec, gint index, GError **err);
-GstAmcBuffer * gst_amc_codec_get_input_buffer (GstAmcCodec * codec, gint index, GError **err);
-
-gint gst_amc_codec_dequeue_input_buffer (GstAmcCodec * codec, gint64 timeoutUs, GError **err);
-gint gst_amc_codec_dequeue_output_buffer (GstAmcCodec * codec, GstAmcBufferInfo *info, gint64 timeoutUs, GError **err);
-
-gboolean gst_amc_codec_queue_input_buffer (GstAmcCodec * codec, gint index, const GstAmcBufferInfo *info, GError **err);
-gboolean gst_amc_codec_release_output_buffer (GstAmcCodec * codec, gint index, gboolean render, GError **err);
-
-
-GstAmcFormat * gst_amc_format_new_audio (const gchar *mime, gint sample_rate, gint channels, GError **err);
-GstAmcFormat * gst_amc_format_new_video (const gchar *mime, gint width, gint height, GError **err);
-void gst_amc_format_free (GstAmcFormat * format);
-
-gchar * gst_amc_format_to_string (GstAmcFormat * format, GError **err);
-
-gboolean gst_amc_format_contains_key (GstAmcFormat *format, const gchar *key, GError **err);
-
-gboolean gst_amc_format_get_float (GstAmcFormat *format, const gchar *key, gfloat *value, GError **err);
-gboolean gst_amc_format_set_float (GstAmcFormat *format, const gchar *key, gfloat value, GError **err);
-gboolean gst_amc_format_get_int (GstAmcFormat *format, const gchar *key, gint *value, GError **err);
-gboolean gst_amc_format_set_int (GstAmcFormat *format, const gchar *key, gint value, GError **err);
-gboolean gst_amc_format_get_string (GstAmcFormat *format, const gchar *key, gchar **value, GError **err);
-gboolean gst_amc_format_set_string (GstAmcFormat *format, const gchar *key, const gchar *value, GError **err);
-gboolean gst_amc_format_get_buffer (GstAmcFormat *format, const gchar *key, guint8 **data, gsize *size, GError **err);
-gboolean gst_amc_format_set_buffer (GstAmcFormat *format, const gchar *key, guint8 *data, gsize size, GError **err);
+gboolean gst_amc_static_init (void);
 
 GstVideoFormat gst_amc_color_format_to_video_format (const GstAmcCodecInfo * codec_info, const gchar * mime, gint color_format);
 gint gst_amc_video_format_to_color_format (const GstAmcCodecInfo * codec_info, const gchar * mime, GstVideoFormat video_format);
