@@ -134,7 +134,7 @@ gst_d3d11_compositor_sizing_policy_get_type (void)
           "padding or keeping the aspect ratio", "none"},
     {GST_D3D11_COMPOSITOR_SIZING_POLICY_KEEP_ASPECT_RATIO,
           "Keep Aspect Ratio: Image is scaled to fit destination rectangle "
-          "specified by GstCompositorPad:{xpos, ypos, width, height} "
+          "specified by GstD3D11CompositorPad:{xpos, ypos, width, height} "
           "with preserved aspect ratio. Resulting image will be centered in "
           "the destination rectangle with padding if necessary",
         "keep-aspect-ratio"},
@@ -150,109 +150,6 @@ gst_d3d11_compositor_sizing_policy_get_type (void)
 }
 
 /* *INDENT-OFF* */
-static const gchar checker_vs_src[] =
-    "struct VS_INPUT\n"
-    "{\n"
-    "  float4 Position : POSITION;\n"
-    "};\n"
-    "\n"
-    "struct VS_OUTPUT\n"
-    "{\n"
-    "  float4 Position: SV_POSITION;\n"
-    "};\n"
-    "\n"
-    "VS_OUTPUT main(VS_INPUT input)\n"
-    "{\n"
-    "  return input;\n"
-    "}\n";
-
-static const gchar checker_ps_src_rgb[] =
-    "static const float blocksize = 8.0;\n"
-    "static const float4 high = float4(0.667, 0.667, 0.667, 1.0);\n"
-    "static const float4 low = float4(0.333, 0.333, 0.333, 1.0);\n"
-    "struct PS_INPUT\n"
-    "{\n"
-    "  float4 Position: SV_POSITION;\n"
-    "};\n"
-    "struct PS_OUTPUT\n"
-    "{\n"
-    "  float4 Plane: SV_TARGET;\n"
-    "};\n"
-    "PS_OUTPUT main(PS_INPUT input)\n"
-    "{\n"
-    "  PS_OUTPUT output;\n"
-    "  if ((input.Position.x % (blocksize * 2.0)) >= blocksize) {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) >= blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  } else {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) < blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  }\n"
-    "  return output;\n"
-    "}\n";
-
-static const gchar checker_ps_src_vuya[] =
-    "static const float blocksize = 8.0;\n"
-    "static const float4 high = float4(0.5, 0.5, 0.667, 1.0);\n"
-    "static const float4 low = float4(0.5, 0.5, 0.333, 1.0);\n"
-    "struct PS_INPUT\n"
-    "{\n"
-    "  float4 Position: SV_POSITION;\n"
-    "};\n"
-    "struct PS_OUTPUT\n"
-    "{\n"
-    "  float4 Plane: SV_TARGET;\n"
-    "};\n"
-    "PS_OUTPUT main(PS_INPUT input)\n"
-    "{\n"
-    "  PS_OUTPUT output;\n"
-    "  if ((input.Position.x % (blocksize * 2.0)) >= blocksize) {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) >= blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  } else {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) < blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  }\n"
-    "  return output;\n"
-    "}\n";
-
-static const gchar checker_ps_src_luma[] =
-    "static const float blocksize = 8.0;\n"
-    "static const float4 high = float4(0.667, 0.0, 0.0, 1.0);\n"
-    "static const float4 low = float4(0.333, 0.0, 0.0, 1.0);\n"
-    "struct PS_INPUT\n"
-    "{\n"
-    "  float4 Position: SV_POSITION;\n"
-    "};\n"
-    "struct PS_OUTPUT\n"
-    "{\n"
-    "  float4 Plane: SV_TARGET;\n"
-    "};\n"
-    "PS_OUTPUT main(PS_INPUT input)\n"
-    "{\n"
-    "  PS_OUTPUT output;\n"
-    "  if ((input.Position.x % (blocksize * 2.0)) >= blocksize) {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) >= blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  } else {\n"
-    "    if ((input.Position.y % (blocksize * 2.0)) < blocksize)\n"
-    "      output.Plane = low;\n"
-    "    else\n"
-    "      output.Plane = high;\n"
-    "  }\n"
-    "  return output;\n"
-    "}\n";
-
 static D3D11_RENDER_TARGET_BLEND_DESC blend_templ[] = {
   /* SOURCE */
   {
@@ -302,6 +199,7 @@ typedef struct
   ID3D11PixelShader *ps;
   ID3D11VertexShader *vs;
   ID3D11InputLayout *layout;
+  ID3D11RasterizerState *rs;
   ID3D11Buffer *vertex_buffer;
   ID3D11Buffer *index_buffer;
   D3D11_VIEWPORT viewport;
@@ -320,6 +218,7 @@ struct _GstD3D11Compositor
   GstD3D11Device *device;
 
   GstBuffer *fallback_buf;
+  GstCaps *negotiated_caps;
 
   GstD3D11CompositorQuad *checker_background;
   /* black/white/transparent */
@@ -356,6 +255,7 @@ enum
 #define DEFAULT_PAD_GAMMA_MODE GST_VIDEO_GAMMA_MODE_NONE
 #define DEFAULT_PAD_PRIMARIES_MODE GST_VIDEO_PRIMARIES_MODE_NONE
 
+static void gst_d3d11_compositor_pad_dispose (GObject * object);
 static void gst_d3d11_compositor_pad_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_d3d11_compositor_pad_get_property (GObject * object,
@@ -380,6 +280,7 @@ gst_d3d11_compositor_pad_class_init (GstD3D11CompositorPadClass * klass)
   GParamFlags param_flags = (GParamFlags)
       (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS);
 
+  object_class->dispose = gst_d3d11_compositor_pad_dispose;
   object_class->set_property = gst_d3d11_compositor_pad_set_property;
   object_class->get_property = gst_d3d11_compositor_pad_get_property;
 
@@ -467,6 +368,17 @@ gst_d3d11_compositor_pad_init (GstD3D11CompositorPad * pad)
   pad->primaries_mode = DEFAULT_PAD_PRIMARIES_MODE;
 }
 
+static void
+gst_d3d11_compositor_pad_dispose (GObject * object)
+{
+  GstD3D11CompositorPad *self = GST_D3D11_COMPOSITOR_PAD (object);
+
+  gst_clear_object (&self->convert);
+  GST_D3D11_CLEAR_COM (self->blend);
+
+  G_OBJECT_CLASS (parent_pad_class)->dispose (object);
+}
+
 static void
 gst_d3d11_compositor_pad_update_position (GstD3D11CompositorPad * self,
     gint * old, const GValue * value)
@@ -752,6 +664,9 @@ gst_d3d11_compositor_pad_check_frame_obscured (GstVideoAggregatorPad * pad,
    *     left unscaled)
    */
 
+  if (cpad->alpha == 0)
+    return TRUE;
+
   gst_d3d11_compositor_pad_get_output_size (cpad, GST_VIDEO_INFO_PAR_N (info),
       GST_VIDEO_INFO_PAR_D (info), &width, &height, &x_offset, &y_offset);
 
@@ -949,6 +864,7 @@ enum
   PROP_0,
   PROP_ADAPTER,
   PROP_BACKGROUND,
+  PROP_IGNORE_INACTIVE_PADS,
 };
 
 #define DEFAULT_ADAPTER -1
@@ -1023,6 +939,26 @@ gst_d3d11_compositor_class_init (GstD3D11CompositorClass * klass)
           DEFAULT_BACKGROUND,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
+  /**
+   * GstD3D11Compositor:ignore-inactive-pads:
+   *
+   * Don't wait for inactive pads when live. An inactive pad
+   * is a pad that hasn't yet received a buffer, but that has
+   * been waited on at least once.
+   *
+   * The purpose of this property is to avoid aggregating on
+   * timeout when new pads are requested in advance of receiving
+   * data flow, for example the user may decide to connect it later,
+   * but wants to configure it already.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (object_class,
+      PROP_IGNORE_INACTIVE_PADS, g_param_spec_boolean ("ignore-inactive-pads",
+          "Ignore inactive pads",
+          "Avoid timing out waiting for inactive pads", FALSE,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+
   element_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_d3d11_compositor_request_new_pad);
   element_class->release_pad =
@@ -1106,6 +1042,10 @@ gst_d3d11_compositor_set_property (GObject * object,
       self->background =
           (GstD3D11CompositorBackground) g_value_get_enum (value);
       break;
+    case PROP_IGNORE_INACTIVE_PADS:
+      gst_aggregator_set_ignore_inactive_pads (GST_AGGREGATOR (object),
+          g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1125,6 +1065,10 @@ gst_d3d11_compositor_get_property (GObject * object,
     case PROP_BACKGROUND:
       g_value_set_enum (value, self->background);
       break;
+    case PROP_IGNORE_INACTIVE_PADS:
+      g_value_set_boolean (value,
+          gst_aggregator_get_ignore_inactive_pads (GST_AGGREGATOR (object)));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1208,15 +1152,12 @@ static void
 gst_d3d11_compositor_release_pad (GstElement * element, GstPad * pad)
 {
   GstD3D11Compositor *self = GST_D3D11_COMPOSITOR (element);
-  GstD3D11CompositorPad *cpad = GST_D3D11_COMPOSITOR_PAD (pad);
 
   GST_DEBUG_OBJECT (self, "Releasing pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
   gst_child_proxy_child_removed (GST_CHILD_PROXY (self), G_OBJECT (pad),
       GST_OBJECT_NAME (pad));
 
-  gst_d3d11_compositor_pad_clear_resource (self, cpad, nullptr);
-
   GST_ELEMENT_CLASS (parent_class)->release_pad (element, pad);
 }
 
@@ -1251,6 +1192,7 @@ gst_d3d11_compositor_stop (GstAggregator * agg)
 
   g_clear_pointer (&self->checker_background, gst_d3d11_compositor_quad_free);
   gst_clear_object (&self->device);
+  gst_clear_caps (&self->negotiated_caps);
 
   return GST_AGGREGATOR_CLASS (parent_class)->stop (agg);
 }
@@ -1616,6 +1558,11 @@ gst_d3d11_compositor_negotiated_src_caps (GstAggregator * agg, GstCaps * caps)
     return FALSE;
   }
 
+  if (self->negotiated_caps && gst_caps_is_equal (self->negotiated_caps, caps)) {
+    GST_DEBUG_OBJECT (self, "Negotiated caps is not changed");
+    goto done;
+  }
+
   features = gst_caps_get_features (caps, 0);
   if (features
       && gst_caps_features_contains (features,
@@ -1672,6 +1619,9 @@ gst_d3d11_compositor_negotiated_src_caps (GstAggregator * agg, GstCaps * caps)
     gst_object_unref (pool);
   }
 
+  gst_caps_replace (&self->negotiated_caps, caps);
+
+done:
   return GST_AGGREGATOR_CLASS (parent_class)->negotiated_src_caps (agg, caps);
 }
 
@@ -1826,14 +1776,11 @@ gst_d3d11_compositor_decide_allocation (GstAggregator * agg, GstQuery * query)
     d3d11_params = gst_buffer_pool_config_get_d3d11_allocation_params (config);
     if (!d3d11_params) {
       d3d11_params = gst_d3d11_allocation_params_new (self->device,
-          &info, GST_D3D11_ALLOCATION_FLAG_DEFAULT, D3D11_BIND_RENDER_TARGET,
-          0);
+          &info, GST_D3D11_ALLOCATION_FLAG_DEFAULT,
+          D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_RENDER_TARGET, 0);
     } else {
-      guint i;
-
-      for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&info); i++) {
-        d3d11_params->desc[i].BindFlags |= D3D11_BIND_RENDER_TARGET;
-      }
+      gst_d3d11_allocation_params_set_bind_flags (d3d11_params,
+          D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_RENDER_TARGET);
     }
 
     gst_buffer_pool_config_set_d3d11_allocation_params (config, d3d11_params);
@@ -1877,76 +1824,46 @@ gst_d3d11_compositor_create_checker_quad (GstD3D11Compositor * self,
     const GstVideoInfo * info)
 {
   GstD3D11CompositorQuad *quad = nullptr;
-  VertexData *vertex_data;
-  WORD *indices;
+  VertexData vertex_data[4];
+  const WORD indices[6] = { 0, 1, 2, 3, 0, 2 };
   ID3D11Device *device_handle;
-  ID3D11DeviceContext *context_handle;
-  D3D11_MAPPED_SUBRESOURCE map;
-  D3D11_INPUT_ELEMENT_DESC input_desc;
   D3D11_BUFFER_DESC buffer_desc;
+  D3D11_SUBRESOURCE_DATA subresource;
   ComPtr < ID3D11Buffer > vertex_buffer;
   ComPtr < ID3D11Buffer > index_buffer;
   ComPtr < ID3D11PixelShader > ps;
   ComPtr < ID3D11VertexShader > vs;
   ComPtr < ID3D11InputLayout > layout;
+  ComPtr < ID3D11RasterizerState > rs;
   HRESULT hr;
-  const gchar *ps_src;
 
   device_handle = gst_d3d11_device_get_device_handle (self->device);
-  context_handle = gst_d3d11_device_get_device_context_handle (self->device);
 
   if (GST_VIDEO_INFO_IS_RGB (info)) {
-    ps_src = checker_ps_src_rgb;
+    hr = gst_d3d11_get_pixel_shader_checker_rgb (self->device, &ps);
   } else if (GST_VIDEO_INFO_FORMAT (info) == GST_VIDEO_FORMAT_VUYA) {
-    ps_src = checker_ps_src_vuya;
+    hr = gst_d3d11_get_pixel_shader_checker_vuya (self->device, &ps);
   } else {
-    ps_src = checker_ps_src_luma;
+    hr = gst_d3d11_get_pixel_shader_checker_luma (self->device, &ps);
   }
 
-  hr = gst_d3d11_create_pixel_shader_simple (self->device, ps_src, "main", &ps);
   if (!gst_d3d11_result (hr, self->device)) {
     GST_ERROR_OBJECT (self, "Couldn't setup pixel shader");
     return nullptr;
   }
 
-  memset (&input_desc, 0, sizeof (D3D11_INPUT_ELEMENT_DESC));
-  input_desc.SemanticName = "POSITION";
-  input_desc.SemanticIndex = 0;
-  input_desc.Format = DXGI_FORMAT_R32G32B32_FLOAT;
-  input_desc.InputSlot = 0;
-  input_desc.AlignedByteOffset = D3D11_APPEND_ALIGNED_ELEMENT;
-  input_desc.InputSlotClass = D3D11_INPUT_PER_VERTEX_DATA;
-  input_desc.InstanceDataStepRate = 0;
-
-  hr = gst_d3d11_create_vertex_shader_simple (self->device, checker_vs_src,
-      "main", &input_desc, 1, &vs, &layout);
+  hr = gst_d3d11_get_vertex_shader_pos (self->device, &vs, &layout);
   if (!gst_d3d11_result (hr, self->device)) {
     GST_ERROR_OBJECT (self, "Couldn't setup vertex shader");
     return nullptr;
   }
 
-  memset (&buffer_desc, 0, sizeof (D3D11_BUFFER_DESC));
-  buffer_desc.Usage = D3D11_USAGE_DYNAMIC;
-  buffer_desc.ByteWidth = sizeof (VertexData) * 4;
-  buffer_desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
-  buffer_desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
-
-  hr = device_handle->CreateBuffer (&buffer_desc, nullptr, &vertex_buffer);
+  hr = gst_d3d11_device_get_rasterizer (self->device, &rs);
   if (!gst_d3d11_result (hr, self->device)) {
-    GST_ERROR_OBJECT (self,
-        "Couldn't create vertex buffer, hr: 0x%x", (guint) hr);
+    GST_ERROR_OBJECT (self, "Couldn't setup rasterizer state");
     return nullptr;
   }
 
-  hr = context_handle->Map (vertex_buffer.Get (),
-      0, D3D11_MAP_WRITE_DISCARD, 0, &map);
-
-  if (!gst_d3d11_result (hr, self->device)) {
-    GST_ERROR_OBJECT (self, "Couldn't map vertex buffer, hr: 0x%x", (guint) hr);
-    return nullptr;
-  }
-
-  vertex_data = (VertexData *) map.pData;
   /* bottom left */
   vertex_data[0].position.x = -1.0f;
   vertex_data[0].position.y = -1.0f;
@@ -1975,44 +1892,45 @@ gst_d3d11_compositor_create_checker_quad (GstD3D11Compositor * self,
   vertex_data[3].texture.u = 1.0f;
   vertex_data[3].texture.v = 1.0f;
 
-  context_handle->Unmap (vertex_buffer.Get (), 0);
+  memset (&subresource, 0, sizeof (D3D11_SUBRESOURCE_DATA));
+  memset (&buffer_desc, 0, sizeof (D3D11_BUFFER_DESC));
+
+  buffer_desc.Usage = D3D11_USAGE_DYNAMIC;
+  buffer_desc.ByteWidth = sizeof (VertexData) * 4;
+  buffer_desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
+  buffer_desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
+
+  subresource.pSysMem = vertex_data;
+  subresource.SysMemPitch = sizeof (VertexData) * 4;
+
+  hr = device_handle->CreateBuffer (&buffer_desc, &subresource, &vertex_buffer);
+  if (!gst_d3d11_result (hr, self->device)) {
+    GST_ERROR_OBJECT (self,
+        "Couldn't create vertex buffer, hr: 0x%x", (guint) hr);
+    return nullptr;
+  }
 
   buffer_desc.Usage = D3D11_USAGE_DYNAMIC;
   buffer_desc.ByteWidth = sizeof (WORD) * 6;
   buffer_desc.BindFlags = D3D11_BIND_INDEX_BUFFER;
   buffer_desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
 
-  hr = device_handle->CreateBuffer (&buffer_desc, nullptr, &index_buffer);
+  subresource.pSysMem = indices;
+  subresource.SysMemPitch = sizeof (WORD) * 6;
+
+  hr = device_handle->CreateBuffer (&buffer_desc, &subresource, &index_buffer);
   if (!gst_d3d11_result (hr, self->device)) {
     GST_ERROR_OBJECT (self,
         "Couldn't create index buffer, hr: 0x%x", (guint) hr);
     return nullptr;
   }
 
-  hr = context_handle->Map (index_buffer.Get (),
-      0, D3D11_MAP_WRITE_DISCARD, 0, &map);
-
-  if (!gst_d3d11_result (hr, self->device)) {
-    GST_ERROR_OBJECT (self, "Couldn't map index buffer, hr: 0x%x", (guint) hr);
-    return nullptr;
-  }
-
-  indices = (WORD *) map.pData;
-
-  /* clockwise indexing */
-  indices[0] = 0;               /* bottom left */
-  indices[1] = 1;               /* top left */
-  indices[2] = 2;               /* top right */
-
-  indices[3] = 3;               /* bottom right */
-  indices[4] = 0;               /* bottom left  */
-  indices[5] = 2;               /* top right */
-
-  context_handle->Unmap (index_buffer.Get (), 0);
   quad = g_new0 (GstD3D11CompositorQuad, 1);
   quad->ps = ps.Detach ();
   quad->vs = vs.Detach ();
+  quad->rs = rs.Detach ();
   quad->layout = layout.Detach ();
+
   quad->vertex_buffer = vertex_buffer.Detach ();
   quad->index_buffer = index_buffer.Detach ();
 
@@ -2035,6 +1953,7 @@ gst_d3d11_compositor_quad_free (GstD3D11CompositorQuad * quad)
   GST_D3D11_CLEAR_COM (quad->ps);
   GST_D3D11_CLEAR_COM (quad->vs);
   GST_D3D11_CLEAR_COM (quad->layout);
+  GST_D3D11_CLEAR_COM (quad->rs);
   GST_D3D11_CLEAR_COM (quad->vertex_buffer);
   GST_D3D11_CLEAR_COM (quad->index_buffer);
 
@@ -2068,6 +1987,7 @@ gst_d3d11_compositor_draw_background_checker (GstD3D11Compositor * self,
   context->VSSetShader (quad->vs, nullptr, 0);
   context->PSSetShader (quad->ps, nullptr, 0);
   context->RSSetViewports (1, &quad->viewport);
+  context->RSSetState (quad->rs);
   context->OMSetRenderTargets (1, &rtv, nullptr);
   context->OMSetBlendState (nullptr, nullptr, 0xffffffff);
   context->DrawIndexed (6, 0, 0);
