@@ -483,7 +483,7 @@ kms_open (gchar ** driver)
   static const char *drivers[] = { "i915", "radeon", "nouveau", "vmwgfx",
     "exynos", "amdgpu", "imx-drm", "imx-lcdif", "rockchip", "atmel-hlcdc",
     "msm", "xlnx", "vc4", "meson", "stm", "sun4i-drm", "mxsfb-drm", "tegra",
-    "xilinx_drm",               /* DEPRECATED. Replaced by xlnx */
+    "tidss", "xilinx_drm",      /* DEPRECATED. Replaced by xlnx */
   };
   int i, fd = -1;
 
