@@ -48,6 +48,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_debug_msdkcontext);
 struct _GstMsdkContextPrivate
 {
   MsdkSession session;
+  GstBufferPool *alloc_pool;
   GList *cached_alloc_responses;
   gboolean hardware;
   gboolean has_frame_allocator;
@@ -167,9 +168,7 @@ static gboolean
 gst_msdk_context_use_vaapi (GstMsdkContext * context)
 {
   char *path;
-  VADisplay va_dpy = NULL;
   GstVaDisplay *display_drm = NULL;
-  mfxStatus status;
   GstMsdkContextPrivate *priv = context->priv;
 
   path = get_device_path ();
@@ -179,31 +178,16 @@ gst_msdk_context_use_vaapi (GstMsdkContext * context)
   }
 
   display_drm = gst_va_display_drm_new_from_path (path);
-  if (!display_drm) {
-    GST_ERROR ("Couldn't create a VA DRM display");
-    goto failed;
-  }
   g_free (path);
 
-  va_dpy = gst_va_display_get_va_dpy (display_drm);
-
-  status = MFXVideoCORE_SetHandle (priv->session.session, MFX_HANDLE_VA_DISPLAY,
-      (mfxHDL) va_dpy);
-  if (status != MFX_ERR_NONE) {
-    GST_ERROR ("Setting VAAPI handle failed (%s)",
-        msdk_status_to_string (status));
-    goto failed;
+  if (!display_drm) {
+    GST_ERROR ("Couldn't create a VA DRM display");
+    return FALSE;
   }
 
   priv->display = display_drm;
 
   return TRUE;
-
-failed:
-  if (display_drm)
-    gst_object_unref (display_drm);
-
-  return FALSE;
 }
 #else
 static GstD3D11Device *
@@ -303,15 +287,17 @@ gst_msdk_context_use_d3d11 (GstMsdkContext * context)
 #endif
 
 static gboolean
-gst_msdk_context_open (GstMsdkContext * context, gboolean hardware,
-    GstMsdkContextJobType job_type)
+gst_msdk_context_open (GstMsdkContext * context, gboolean hardware)
 {
   mfxU16 codename;
   GstMsdkContextPrivate *priv = context->priv;
   MsdkSession msdk_session;
   mfxIMPL impl;
+  mfxHDL handle = NULL;
+#ifndef _WIN32
+  mfxStatus status;
+#endif
 
-  priv->job_type = job_type;
   priv->hardware = hardware;
 
   impl = hardware ? MFX_IMPL_HARDWARE_ANY : MFX_IMPL_SOFTWARE;
@@ -320,23 +306,36 @@ gst_msdk_context_open (GstMsdkContext * context, gboolean hardware,
   impl |= MFX_IMPL_VIA_D3D11;
 #endif
 
-  msdk_session = msdk_open_session (impl);
-  priv->session = msdk_session;
-  if (!priv->session.session)
-    goto failed;
-
 #ifndef _WIN32
   if (hardware) {
     if (!gst_msdk_context_use_vaapi (context))
-      goto failed;
-  }
-#else
-  if (hardware) {
-    if (!gst_msdk_context_use_d3d11 (context))
-      goto failed;
+      return FALSE;
+
+    handle = (mfxHDL) gst_va_display_get_va_dpy (priv->display);
   }
 #endif
 
+  msdk_session = msdk_open_session (handle, impl);
+  if (!msdk_session.session)
+    return FALSE;
+
+  priv->session = msdk_session;
+
+  if (hardware) {
+#ifndef _WIN32
+    status = MFXVideoCORE_SetHandle (priv->session.session,
+        MFX_HANDLE_VA_DISPLAY, handle);
+    if (status != MFX_ERR_NONE) {
+      GST_ERROR ("Setting VAAPI handle failed (%s)",
+          msdk_status_to_string (status));
+      return FALSE;
+    }
+#else
+    if (!gst_msdk_context_use_d3d11 (context))
+      return FALSE;
+#endif
+  }
+
   codename = msdk_get_platform_codename (priv->session.session);
 
   if (codename != MFX_PLATFORM_UNKNOWN)
@@ -345,9 +344,6 @@ gst_msdk_context_open (GstMsdkContext * context, gboolean hardware,
     GST_WARNING ("Unknown MFX platform");
 
   return TRUE;
-
-failed:
-  return FALSE;
 }
 
 static void
@@ -409,32 +405,47 @@ gst_msdk_context_class_init (GstMsdkContextClass * klass)
 }
 
 GstMsdkContext *
-gst_msdk_context_new (gboolean hardware, GstMsdkContextJobType job_type)
+gst_msdk_context_new (gboolean hardware)
 {
   GstMsdkContext *obj = g_object_new (GST_TYPE_MSDK_CONTEXT, NULL);
+  gst_object_ref_sink (obj);
 
-  if (obj && !gst_msdk_context_open (obj, hardware, job_type)) {
-    if (obj)
-      gst_object_unref (obj);
+  if (obj && !gst_msdk_context_open (obj, hardware)) {
+    gst_object_unref (obj);
     return NULL;
   }
 
   return obj;
 }
 
+GstMsdkContext *
+gst_msdk_context_new_with_job_type (gboolean hardware,
+    GstMsdkContextJobType job_type)
+{
+  GstMsdkContext *obj = gst_msdk_context_new (hardware);
+
+  if (obj)
+    obj->priv->job_type = job_type;
+
+  return obj;
+}
+
 GstMsdkContext *
 gst_msdk_context_new_with_parent (GstMsdkContext * parent)
 {
   mfxStatus status;
-  GstMsdkContext *obj = g_object_new (GST_TYPE_MSDK_CONTEXT, NULL);
-  GstMsdkContextPrivate *priv = obj->priv;
-  GstMsdkContextPrivate *parent_priv = parent->priv;
+  GstMsdkContext *obj;
+  GstMsdkContextPrivate *priv;
+  GstMsdkContextPrivate *parent_priv;
   mfxVersion version;
   mfxIMPL impl;
   MsdkSession child_msdk_session;
   mfxHandleType handle_type = 0;
-  mfxHDL handle = NULL;
+  mfxHDL handle = NULL, hardware_handle = NULL;
 
+  g_return_val_if_fail (GST_IS_MSDK_CONTEXT (parent), NULL);
+
+  parent_priv = parent->priv;
   status = MFXQueryIMPL (parent_priv->session.session, &impl);
 
   if (status == MFX_ERR_NONE)
@@ -443,7 +454,6 @@ gst_msdk_context_new_with_parent (GstMsdkContext * parent)
   if (status != MFX_ERR_NONE) {
     GST_ERROR ("Failed to query the session attributes (%s)",
         msdk_status_to_string (status));
-    gst_object_unref (obj);
     return NULL;
   }
 
@@ -460,19 +470,22 @@ gst_msdk_context_new_with_parent (GstMsdkContext * parent)
     if (status != MFX_ERR_NONE || !handle) {
       GST_ERROR ("Failed to get session handle (%s)",
           msdk_status_to_string (status));
-      gst_object_unref (obj);
       return NULL;
     }
   }
 
   child_msdk_session.loader = parent_priv->session.loader;
   child_msdk_session.session = NULL;
-  status = msdk_init_msdk_session (impl, &version, &child_msdk_session);
+#ifndef _WIN32
+  hardware_handle = (mfxHDL) gst_va_display_get_va_dpy (parent_priv->display);
+#endif
+
+  status = msdk_init_msdk_session (hardware_handle, impl, &version,
+      &child_msdk_session);
 
   if (status != MFX_ERR_NONE) {
     GST_ERROR ("Failed to create a child mfx session (%s)",
         msdk_status_to_string (status));
-    gst_object_unref (obj);
     return NULL;
   }
 
@@ -485,7 +498,6 @@ gst_msdk_context_new_with_parent (GstMsdkContext * parent)
       GST_ERROR ("Failed to set a HW handle (%s)",
           msdk_status_to_string (status));
       MFXClose (child_msdk_session.session);
-      gst_object_unref (obj);
       return NULL;
     }
   }
@@ -497,11 +509,14 @@ gst_msdk_context_new_with_parent (GstMsdkContext * parent)
     GST_ERROR ("Failed to join two sessions (%s)",
         msdk_status_to_string (status));
     MFXClose (child_msdk_session.session);
-    gst_object_unref (obj);
     return NULL;
   }
 #endif
 
+  obj = g_object_new (GST_TYPE_MSDK_CONTEXT, NULL);
+  gst_object_ref_sink (obj);
+  priv = obj->priv;
+
   /* Set loader to NULL for child session */
   priv->session.loader = NULL;
   priv->session.session = child_msdk_session.session;
@@ -530,20 +545,24 @@ gst_msdk_context_new_with_va_display (GstObject * display_obj,
   mfxU16 codename;
   mfxStatus status;
   GstVaDisplay *va_display;
+  mfxHDL handle;
 
   va_display = GST_VA_DISPLAY (display_obj);
   if (!va_display)
     return NULL;
 
   obj = g_object_new (GST_TYPE_MSDK_CONTEXT, NULL);
+  gst_object_ref_sink (obj);
 
   priv = obj->priv;
   priv->display = gst_object_ref (va_display);
 
   priv->job_type = job_type;
   priv->hardware = hardware;
-  priv->session =
-      msdk_open_session (hardware ? MFX_IMPL_HARDWARE_ANY : MFX_IMPL_SOFTWARE);
+
+  handle = (mfxHDL) gst_va_display_get_va_dpy (priv->display);
+  priv->session = msdk_open_session (handle,
+      hardware ? MFX_IMPL_HARDWARE_ANY : MFX_IMPL_SOFTWARE);
   if (!priv->session.session) {
     gst_object_unref (obj);
     return NULL;
@@ -552,7 +571,7 @@ gst_msdk_context_new_with_va_display (GstObject * display_obj,
   if (hardware) {
     status =
         MFXVideoCORE_SetHandle (priv->session.session, MFX_HANDLE_VA_DISPLAY,
-        (mfxHDL) gst_va_display_get_va_dpy (priv->display));
+        handle);
     if (status != MFX_ERR_NONE) {
       GST_ERROR ("Setting VAAPI handle failed (%s)",
           msdk_status_to_string (status));
@@ -584,14 +603,15 @@ gst_msdk_context_new_with_d3d11_device (GstD3D11Device * device,
   HRESULT hr;
 
   obj = g_object_new (GST_TYPE_MSDK_CONTEXT, NULL);
+  gst_object_ref_sink (obj);
 
   priv = obj->priv;
   priv->device = gst_object_ref (device);
 
   priv->job_type = job_type;
   priv->hardware = hardware;
-  priv->session =
-      msdk_open_session (hardware ? MFX_IMPL_HARDWARE_ANY : MFX_IMPL_SOFTWARE);
+  priv->session = msdk_open_session (NULL,
+      hardware ? MFX_IMPL_HARDWARE_ANY : MFX_IMPL_SOFTWARE);
   if (!priv->session.session) {
     goto failed;
   }
@@ -640,6 +660,18 @@ gst_msdk_context_get_session (GstMsdkContext * context)
   return context->priv->session.session;
 }
 
+const mfxLoader *
+gst_msdk_context_get_loader (GstMsdkContext * context)
+{
+  return &context->priv->session.loader;
+}
+
+mfxU32
+gst_msdk_context_get_impl_idx (GstMsdkContext * context)
+{
+  return context->priv->session.impl_idx;
+}
+
 gpointer
 gst_msdk_context_get_handle (GstMsdkContext * context)
 {
@@ -735,47 +767,12 @@ gst_msdk_context_get_cached_alloc_responses_by_request (GstMsdkContext *
     return NULL;
 }
 
-static void
-create_surfaces (GstMsdkContext * context, GstMsdkAllocResponse * resp)
-{
-  gint i;
-  mfxMemId *mem_id;
-  mfxFrameSurface1 *surface;
-
-  for (i = 0; i < resp->response.NumFrameActual; i++) {
-    mem_id = resp->response.mids[i];
-    surface = (mfxFrameSurface1 *) g_slice_new0 (mfxFrameSurface1);
-    if (!surface) {
-      GST_ERROR ("failed to allocate surface");
-      break;
-    }
-    surface->Data.MemId = mem_id;
-    resp->surfaces_avail = g_list_prepend (resp->surfaces_avail, surface);
-  }
-}
-
-static void
-free_surface (gpointer surface)
-{
-  g_slice_free1 (sizeof (mfxFrameSurface1), surface);
-}
-
-static void
-remove_surfaces (GstMsdkContext * context, GstMsdkAllocResponse * resp)
-{
-  g_list_free_full (resp->surfaces_used, free_surface);
-  g_list_free_full (resp->surfaces_avail, free_surface);
-  g_list_free_full (resp->surfaces_locked, free_surface);
-}
-
 void
 gst_msdk_context_add_alloc_response (GstMsdkContext * context,
     GstMsdkAllocResponse * resp)
 {
   context->priv->cached_alloc_responses =
       g_list_prepend (context->priv->cached_alloc_responses, resp);
-
-  create_surfaces (context, resp);
 }
 
 gboolean
@@ -792,8 +789,6 @@ gst_msdk_context_remove_alloc_response (GstMsdkContext * context,
 
   msdk_resp = l->data;
 
-  remove_surfaces (context, msdk_resp);
-
   g_slice_free1 (sizeof (GstMsdkAllocResponse), msdk_resp);
   priv->cached_alloc_responses =
       g_list_delete_link (priv->cached_alloc_responses, l);
@@ -801,120 +796,16 @@ gst_msdk_context_remove_alloc_response (GstMsdkContext * context,
   return TRUE;
 }
 
-static gboolean
-check_surfaces_available (GstMsdkContext * context, GstMsdkAllocResponse * resp)
-{
-  GList *l;
-  mfxFrameSurface1 *surface = NULL;
-  GstMsdkContextPrivate *priv = context->priv;
-  gboolean ret = FALSE;
-
-  g_mutex_lock (&priv->mutex);
-  for (l = resp->surfaces_locked; l;) {
-    surface = l->data;
-    l = l->next;
-    if (!surface->Data.Locked) {
-      resp->surfaces_locked = g_list_remove (resp->surfaces_locked, surface);
-      resp->surfaces_avail = g_list_prepend (resp->surfaces_avail, surface);
-      ret = TRUE;
-    }
-  }
-  g_mutex_unlock (&priv->mutex);
-
-  return ret;
-}
-
-/*
- * There are 3 lists here in GstMsdkContext as the following:
- * 1. surfaces_avail : surfaces which are free and unused anywhere
- * 2. surfaces_used : surfaces coupled with a gst buffer and being used now.
- * 3. surfaces_locked : surfaces still locked even after the gst buffer is released.
- *
- * Note that they need to be protected by mutex to be thread-safe.
- */
-
-mfxFrameSurface1 *
-gst_msdk_context_get_surface_available (GstMsdkContext * context,
-    mfxFrameAllocResponse * resp)
-{
-  GList *l;
-  mfxFrameSurface1 *surface = NULL;
-  GstMsdkAllocResponse *msdk_resp =
-      gst_msdk_context_get_cached_alloc_responses (context, resp);
-  gint retry = 0;
-  GstMsdkContextPrivate *priv = context->priv;
-
-retry:
-  g_mutex_lock (&priv->mutex);
-  for (l = msdk_resp->surfaces_avail; l;) {
-    surface = l->data;
-    l = l->next;
-    if (!surface->Data.Locked) {
-      msdk_resp->surfaces_avail =
-          g_list_remove (msdk_resp->surfaces_avail, surface);
-      msdk_resp->surfaces_used =
-          g_list_prepend (msdk_resp->surfaces_used, surface);
-      break;
-    }
-  }
-  g_mutex_unlock (&priv->mutex);
-
-  /*
-   * If a msdk context is shared by multiple msdk elements,
-   * upstream msdk element sometimes needs to wait for a gst buffer
-   * to be released in downstream.
-   *
-   * Poll the pool for a maximum of 20 millisecond.
-   *
-   * FIXME: Is there any better way to handle this case?
-   */
-  if (!surface && retry < 20) {
-    /* If there's no surface available, find unlocked surfaces in the locked list,
-     * take it back to the available list and then search again.
-     */
-    check_surfaces_available (context, msdk_resp);
-    retry++;
-    g_usleep (1000);
-    goto retry;
-  }
-
-  return surface;
-}
-
 void
-gst_msdk_context_put_surface_locked (GstMsdkContext * context,
-    mfxFrameAllocResponse * resp, mfxFrameSurface1 * surface)
+gst_msdk_context_set_alloc_pool (GstMsdkContext * context, GstBufferPool * pool)
 {
-  GstMsdkContextPrivate *priv = context->priv;
-  GstMsdkAllocResponse *msdk_resp =
-      gst_msdk_context_get_cached_alloc_responses (context, resp);
-
-  g_mutex_lock (&priv->mutex);
-  if (!g_list_find (msdk_resp->surfaces_locked, surface)) {
-    msdk_resp->surfaces_used =
-        g_list_remove (msdk_resp->surfaces_used, surface);
-    msdk_resp->surfaces_locked =
-        g_list_prepend (msdk_resp->surfaces_locked, surface);
-  }
-  g_mutex_unlock (&priv->mutex);
+  context->priv->alloc_pool = gst_object_ref (pool);
 }
 
-void
-gst_msdk_context_put_surface_available (GstMsdkContext * context,
-    mfxFrameAllocResponse * resp, mfxFrameSurface1 * surface)
+GstBufferPool *
+gst_msdk_context_get_alloc_pool (GstMsdkContext * context)
 {
-  GstMsdkContextPrivate *priv = context->priv;
-  GstMsdkAllocResponse *msdk_resp =
-      gst_msdk_context_get_cached_alloc_responses (context, resp);
-
-  g_mutex_lock (&priv->mutex);
-  if (!g_list_find (msdk_resp->surfaces_avail, surface)) {
-    msdk_resp->surfaces_used =
-        g_list_remove (msdk_resp->surfaces_used, surface);
-    msdk_resp->surfaces_avail =
-        g_list_prepend (msdk_resp->surfaces_avail, surface);
-  }
-  g_mutex_unlock (&priv->mutex);
+  return context->priv->alloc_pool;
 }
 
 GstMsdkContextJobType
@@ -923,6 +814,13 @@ gst_msdk_context_get_job_type (GstMsdkContext * context)
   return context->priv->job_type;
 }
 
+void
+gst_msdk_context_set_job_type (GstMsdkContext * context,
+    GstMsdkContextJobType job_type)
+{
+  context->priv->job_type = job_type;
+}
+
 void
 gst_msdk_context_add_job_type (GstMsdkContext * context,
     GstMsdkContextJobType job_type)
