@@ -56,28 +56,25 @@
 GST_DEBUG_CATEGORY_EXTERN (gst_msdkvc1dec_debug);
 #define GST_CAT_DEFAULT gst_msdkvc1dec_debug
 
-static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-wmv, "
-        "framerate = (fraction) [0/1, MAX], "
-        "width = (int) [ 1, MAX ], height = (int) [ 1, MAX ], "
-        "wmvversion= (int) 3, "
-        "format= (string) WMV3, "
-        "header-format= (string) none, "
-        "stream-format= (string) sequence-layer-frame-layer, "
-        "profile = (string) {simple, main}" ";"
-        "video/x-wmv, "
-        "framerate = (fraction) [0/1, MAX], "
-        "width = (int) [ 1, MAX ], height = (int) [ 1, MAX ], "
-        "wmvversion= (int) 3, "
-        "format= (string) WVC1, "
-        "header-format= (string) asf, "
-        "stream-format= (string) bdu, " "profile = (string) advanced" ";")
-    );
+#define GST_MSDKVC1DEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_FROM_INSTANCE (obj), GstMsdkVC1Dec))
+#define GST_MSDKVC1DEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_FROM_CLASS (klass), GstMsdkVC1DecClass))
+#define GST_IS_MSDKVC1DEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_FROM_INSTANCE (obj)))
+#define GST_IS_MSDKVC1DEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_FROM_CLASS (klass)))
 
-#define gst_msdkvc1dec_parent_class parent_class
-G_DEFINE_TYPE (GstMsdkVC1Dec, gst_msdkvc1dec, GST_TYPE_MSDKDEC);
+/* *INDENT-OFF* */
+static const gchar *doc_src_caps_str =
+    GST_VIDEO_CAPS_MAKE ("{ NV12 }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:DMABuf", "{ NV12 }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:VAMemory", "{ NV12 }");
+/* *INDENT-ON* */
+
+static const gchar *doc_sink_caps_str = "video/x-wmv";
+
+static GstElementClass *parent_class = NULL;
 
 static gboolean
 gst_msdkvc1dec_configure (GstMsdkDec * decoder)
@@ -183,11 +180,14 @@ gst_msdkvc1dec_preinit_decoder (GstMsdkDec * decoder)
 }
 
 static void
-gst_msdkvc1dec_class_init (GstMsdkVC1DecClass * klass)
+gst_msdkvc1dec_class_init (gpointer klass, gpointer data)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class;
   GstMsdkDecClass *decoder_class;
+  MsdkDecCData *cdata = data;
+
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
@@ -208,11 +208,59 @@ gst_msdkvc1dec_class_init (GstMsdkVC1DecClass * klass)
 
   gst_msdkdec_prop_install_output_oder_property (gobject_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_msdkcaps_pad_template_init (element_class,
+      cdata->sink_caps, cdata->src_caps, doc_sink_caps_str, doc_src_caps_str);
+
+  gst_caps_unref (cdata->sink_caps);
+  gst_caps_unref (cdata->src_caps);
+  g_free (cdata);
+
 }
 
 static void
-gst_msdkvc1dec_init (GstMsdkVC1Dec * thiz)
+gst_msdkvc1dec_init (GTypeInstance * instance, gpointer g_class)
 {
+  GstMsdkVC1Dec *thiz = GST_MSDKVC1DEC (instance);
   thiz->output_order = PROP_OUTPUT_ORDER_DEFAULT;
 }
+
+gboolean
+gst_msdkvc1dec_register (GstPlugin * plugin,
+    GstMsdkContext * context, GstCaps * sink_caps,
+    GstCaps * src_caps, guint rank)
+{
+  GType type;
+  MsdkDecCData *cdata;
+  gchar *type_name, *feature_name;
+  gboolean ret = FALSE;
+
+  GTypeInfo type_info = {
+    .class_size = sizeof (GstMsdkVC1DecClass),
+    .class_init = gst_msdkvc1dec_class_init,
+    .instance_size = sizeof (GstMsdkVC1Dec),
+    .instance_init = gst_msdkvc1dec_init
+  };
+
+  cdata = g_new (MsdkDecCData, 1);
+  cdata->sink_caps = gst_caps_ref (sink_caps);
+  cdata->src_caps = gst_caps_ref (src_caps);
+
+  GST_MINI_OBJECT_FLAG_SET (cdata->sink_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+  GST_MINI_OBJECT_FLAG_SET (cdata->src_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+
+  type_info.class_data = cdata;
+
+  type_name = g_strdup ("GstMsdkVC1Dec");
+  feature_name = g_strdup ("msdkvc1dec");
+
+  type = g_type_register_static (GST_TYPE_MSDKDEC, type_name, &type_info, 0);
+  if (type)
+    ret = gst_element_register (plugin, feature_name, rank, type);
+
+  g_free (type_name);
+  g_free (feature_name);
+
+  return ret;
+}
