@@ -71,14 +71,18 @@ static const struct fourcc_map gst_msdk_fourcc_mfx_to_va[] = {
   FOURCC_MFX_TO_VA (RGB565, RGB565),
 #endif
   FOURCC_MFX_TO_VA (AYUV, AYUV),
-#if VA_CHECK_VERSION(1, 4, 1)
   FOURCC_MFX_TO_VA (A2RGB10, A2R10G10B10),
-#endif
-#if ((MFX_VERSION >= 1027) && VA_CHECK_VERSION(1, 2, 0))
+#if (MFX_VERSION >= 1027)
   FOURCC_MFX_TO_VA (Y210, Y210),
   FOURCC_MFX_TO_VA (Y410, Y410),
 #endif
   FOURCC_MFX_TO_VA (BGR4, ABGR),
+
+#if (MFX_VERSION >= 2004)
+  FOURCC_MFX_TO_VA (RGBP, RGBP),
+  FOURCC_MFX_TO_VA (BGRP, BGRP),
+#endif
+
   {0, 0}
 };
 
