@@ -30,7 +30,7 @@
 
 #include <gmodule.h>
 
-#if HAVE_NVCODEC_GST_GL
+#ifdef HAVE_CUDA_GST_GL
 #include <gst/gl/gl.h>
 #endif
 
@@ -150,6 +150,18 @@ NvEncGetEncodePresetConfig (void *encoder, GUID encodeGUID,
       presetConfig);
 }
 
+NVENCSTATUS NVENCAPI
+NvEncGetEncodePresetConfigEx (void *encoder, GUID encodeGUID,
+    GUID presetGUID, NV_ENC_TUNING_INFO tuningInfo,
+    NV_ENC_PRESET_CONFIG * presetConfig)
+{
+  if (!nvenc_api.nvEncGetEncodePresetConfigEx)
+    return NV_ENC_ERR_UNIMPLEMENTED;
+
+  return nvenc_api.nvEncGetEncodePresetConfigEx (encoder, encodeGUID,
+      presetGUID, tuningInfo, presetConfig);
+}
+
 NVENCSTATUS NVENCAPI
 NvEncGetEncodeCaps (void *encoder, GUID encodeGUID,
     NV_ENC_CAPS_PARAM * capsParam, int *capsVal)
@@ -827,7 +839,7 @@ gst_nv_enc_register (GstPlugin * plugin, GUID codec_id, const gchar * codec,
 
     {
       GstCaps *cuda_caps = gst_caps_copy (sink_templ);
-#if HAVE_NVCODEC_GST_GL
+#ifdef HAVE_CUDA_GST_GL
       GstCaps *gl_caps = gst_caps_copy (sink_templ);
       gst_caps_set_features_simple (gl_caps,
           gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
