@@ -94,11 +94,10 @@ gst_qsv_h265_enc_sei_insert_mode_get_type (void)
     {0, nullptr, nullptr}
   };
 
-  if (g_once_init_enter (&sei_insert_mode_type)) {
-    GType type =
+  GST_QSV_CALL_ONCE_BEGIN {
+    sei_insert_mode_type =
         g_enum_register_static ("GstQsvH265EncSeiInsertMode", insert_modes);
-    g_once_init_leave (&sei_insert_mode_type, type);
-  }
+  } GST_QSV_CALL_ONCE_END;
 
   return sei_insert_mode_type;
 }
@@ -158,11 +157,10 @@ gst_qsv_h265_enc_rate_control_get_type (void)
     {0, nullptr, nullptr}
   };
 
-  if (g_once_init_enter (&rate_control_type)) {
-    GType type =
+  GST_QSV_CALL_ONCE_BEGIN {
+    rate_control_type =
         g_enum_register_static ("GstQsvH265EncRateControl", rate_controls);
-    g_once_init_leave (&rate_control_type, type);
-  }
+  } GST_QSV_CALL_ONCE_END;
 
   return rate_control_type;
 }
@@ -540,13 +538,11 @@ gst_qsv_h265_enc_check_update_uint (GstQsvH265Enc * self, guint * old_val,
   if (*old_val == new_val)
     return;
 
-  g_mutex_lock (&self->prop_lock);
   *old_val = new_val;
   if (is_bitrate_param)
     self->bitrate_updated = TRUE;
   else
     self->property_updated = TRUE;
-  g_mutex_unlock (&self->prop_lock);
 }
 
 static void
@@ -556,10 +552,8 @@ gst_qsv_h265_enc_check_update_enum (GstQsvH265Enc * self, mfxU16 * old_val,
   if (*old_val == (mfxU16) new_val)
     return;
 
-  g_mutex_lock (&self->prop_lock);
   *old_val = (mfxU16) new_val;
   self->property_updated = TRUE;
-  g_mutex_unlock (&self->prop_lock);
 }
 
 static void
@@ -569,10 +563,8 @@ gst_qsv_h265_enc_check_update_boolean (GstQsvH265Enc * self, gboolean * old_val,
   if (*old_val == new_val)
     return;
 
-  g_mutex_lock (&self->prop_lock);
   *old_val = new_val;
   self->property_updated = TRUE;
-  g_mutex_unlock (&self->prop_lock);
 }
 
 static void
@@ -581,6 +573,7 @@ gst_qsv_h265_enc_set_property (GObject * object, guint prop_id,
 {
   GstQsvH265Enc *self = GST_QSV_H265_ENC (object);
 
+  g_mutex_lock (&self->prop_lock);
   switch (prop_id) {
     case PROP_MIN_QP_I:
       gst_qsv_h265_enc_check_update_uint (self, &self->min_qp_i,
@@ -666,6 +659,7 @@ gst_qsv_h265_enc_set_property (GObject * object, guint prop_id,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
+  g_mutex_unlock (&self->prop_lock);
 }
 
 static void
@@ -674,6 +668,7 @@ gst_qsv_h265_enc_get_property (GObject * object, guint prop_id, GValue * value,
 {
   GstQsvH265Enc *self = GST_QSV_H265_ENC (object);
 
+  g_mutex_lock (&self->prop_lock);
   switch (prop_id) {
     case PROP_MIN_QP_I:
       g_value_set_uint (value, self->min_qp_i);
@@ -739,6 +734,7 @@ gst_qsv_h265_enc_get_property (GObject * object, guint prop_id, GValue * value,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
+  g_mutex_unlock (&self->prop_lock);
 }
 
 static gboolean
@@ -1464,29 +1460,23 @@ gst_qsv_h265_enc_register (GstPlugin * plugin, guint rank, guint impl_index,
 
   /* Check supported profiles */
   for (guint i = 0; i < G_N_ELEMENTS (profile_map); i++) {
+    GstVideoFormat format = GST_VIDEO_FORMAT_UNKNOWN;
     mfx->CodecProfile = profile_map[i].profile;
     mfx->CodecLevel = MFX_LEVEL_UNKNOWN;
 
     switch (mfx->CodecProfile) {
       case MFX_PROFILE_HEVC_MAIN:
-        mfx->FrameInfo.ChromaFormat = MFX_CHROMAFORMAT_YUV420;
-        mfx->FrameInfo.FourCC = MFX_FOURCC_NV12;
-        mfx->FrameInfo.BitDepthLuma = 8;
-        mfx->FrameInfo.BitDepthChroma = 8;
-        mfx->FrameInfo.Shift = 0;
+        format = GST_VIDEO_FORMAT_NV12;
         break;
       case MFX_PROFILE_HEVC_MAIN10:
-        mfx->FrameInfo.ChromaFormat = MFX_CHROMAFORMAT_YUV420;
-        mfx->FrameInfo.FourCC = MFX_FOURCC_P010;
-        mfx->FrameInfo.BitDepthLuma = 10;
-        mfx->FrameInfo.BitDepthChroma = 10;
-        mfx->FrameInfo.Shift = 1;
+        format = GST_VIDEO_FORMAT_P010_10LE;
         break;
       default:
         g_assert_not_reached ();
         return;
     }
 
+    gst_qsv_frame_info_set_format (&mfx->FrameInfo, format);
     if (MFXVideoENCODE_Query (session, &param, &param) != MFX_ERR_NONE)
       continue;
 
@@ -1499,11 +1489,7 @@ gst_qsv_h265_enc_register (GstPlugin * plugin, guint rank, guint impl_index,
     return;
   }
 
-  mfx->FrameInfo.ChromaFormat = MFX_CHROMAFORMAT_YUV420;
-  mfx->FrameInfo.FourCC = MFX_FOURCC_NV12;
-  mfx->FrameInfo.BitDepthLuma = 8;
-  mfx->FrameInfo.BitDepthChroma = 8;
-  mfx->FrameInfo.Shift = 0;
+  gst_qsv_frame_info_set_format (&mfx->FrameInfo, GST_VIDEO_FORMAT_NV12);
   mfx->CodecProfile = MFX_PROFILE_HEVC_MAIN;
 
   /* check hdr10 metadata SEI support */
