@@ -338,6 +338,14 @@ gst_va_base_transform_decide_allocation (GstBaseTransform * trans,
   if (!outcaps)
     return FALSE;
 
+  has_videometa = gst_query_find_allocation_meta (query,
+      GST_VIDEO_META_API_TYPE, NULL);
+  if (gst_video_is_dma_drm_caps (outcaps) && !has_videometa) {
+    GST_ERROR_OBJECT (trans,
+        "DMABuf caps negotiated without the mandatory support of VideoMeta ");
+    return FALSE;
+  }
+
   if (gst_query_get_n_allocation_params (query) > 0) {
     GstVaDisplay *display;
 
@@ -423,9 +431,6 @@ gst_va_base_transform_decide_allocation (GstBaseTransform * trans,
   else
     gst_query_add_allocation_pool (query, pool, size, min, max);
 
-  has_videometa = gst_query_find_allocation_meta (query,
-      GST_VIDEO_META_API_TYPE, NULL);
-
   copy_frames = (!has_videometa && gst_va_pool_requires_video_meta (pool)
       && gst_caps_is_raw (outcaps));
   if (copy_frames) {
