@@ -375,7 +375,7 @@ gst_va_vp8_dec_end_picture (GstVp8Decoder * decoder, GstVp8Picture * picture)
   GstVaDecodePicture *va_pic;
 
   GST_LOG_OBJECT (base, "end picture %p, (system_frame_number %d)",
-      picture, picture->system_frame_number);
+      picture, GST_CODEC_PICTURE (picture)->system_frame_number);
 
   va_pic = gst_vp8_picture_get_user_data (picture);
 
@@ -392,13 +392,15 @@ gst_va_vp8_dec_output_picture (GstVp8Decoder * decoder,
   GstVaBaseDec *base = GST_VA_BASE_DEC (decoder);
   GstVaVp8Dec *self = GST_VA_VP8_DEC (decoder);
   GstVideoDecoder *vdec = GST_VIDEO_DECODER (decoder);
+  GstCodecPicture *codec_picture = GST_CODEC_PICTURE (picture);
   gboolean ret;
 
   GST_LOG_OBJECT (self,
       "Outputting picture %p (system_frame_number %d)",
-      picture, picture->system_frame_number);
+      picture, codec_picture->system_frame_number);
 
-  ret = gst_va_base_dec_process_output (base, frame, picture->discont_state, 0);
+  ret = gst_va_base_dec_process_output (base, frame,
+      codec_picture->discont_state, 0);
   gst_vp8_picture_unref (picture);
 
   if (ret)
@@ -520,22 +522,9 @@ gst_va_vp8_dec_register (GstPlugin * plugin, GstVaDevice * device,
 
   type_info.class_data = cdata;
 
-  /* The first decoder to be registered should use a constant name,
-   * like vavp8dec, for any additional decoders, we create unique
-   * names, using inserting the render device name. */
-  if (device->index == 0) {
-    type_name = g_strdup ("GstVaVp8Dec");
-    feature_name = g_strdup ("vavp8dec");
-  } else {
-    gchar *basename = g_path_get_basename (device->render_device_path);
-    type_name = g_strdup_printf ("GstVa%sVp8Dec", basename);
-    feature_name = g_strdup_printf ("va%svp8dec", basename);
-    cdata->description = basename;
-
-    /* lower rank for non-first device */
-    if (rank > 0)
-      rank--;
-  }
+  gst_va_create_feature_name (device, "GstVaVp8Dec", "GstVa%sVp8Dec",
+      &type_name, "vavp8dec", "va%svp8dec", &feature_name,
+      &cdata->description, &rank);
 
   g_once (&debug_once, _register_debug_category, NULL);
 
